<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Concerns\WithEvents;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Events\AfterSheet;
use App\Models\Department;
use App\Models\User;
use App\Models\SubjectType;
use App\Models\StaffSubject;
use App\Models\Subject; 

class StaffSubjectExport implements FromCollection, WithHeadings, WithTitle, WithStyles, WithEvents
{
    /**
    * @return \Illuminate\Support\Collection
    */
    protected $usertype;
    protected $ac_id;
     public function __construct($usertype,$dept_id, $ac_id)
    {
        $this->utype = $usertype;
        $this->dept_id = $dept_id;
        $this->acid = $ac_id;
      
    }
    public function collection()
    {
        $query = StaffSubject::query()
                    ->leftJoin('users', 'staff_subject.user_id', '=', 'users.user_id')
                    ->leftJoin('subjects', 'staff_subject.subject_id', '=', 'subjects.subject_id')
                    ->leftJoin('sub_category', 'subjects.subject_categ', '=', 'sub_category.id')
                  ->leftJoin('courses', 'staff_subject.course_id', '=', 'courses.course_id')
                  ->leftJoin('semesters', 'staff_subject.semester_id', '=', 'semesters.semester_id')
                  ->leftJoin('deparment', 'staff_subject.semester_id', '=', 'semesters.semester_id')
                  
                    ->whereNull('staff_subject.deleted_at');
                    if($this->acid!=0){
            $query->where('staff_subject.acyear_id', $this->acid);
                    }
                      if($this->utype == 17){
            $query->where('subjects.department_id', $this->dept_id);
            $query->where('users.dept_id', $this->dept_id);
                    }
                    $query->groupBy(
        'staff_subject.user_id', // Grouping by user
        'staff_subject.subject_id', // Grouping by subject
        'staff_subject.course_id', // Grouping by course
        'staff_subject.semester_id', // Grouping by semester
        'subjects.subject_name',
        'subjects.subject_code',
        'sub_category.category',
        'users.name',
        'users.lname',
        'courses.course_name',
        'semesters.semester_name'
    );
             $staffsubjects = $query->select(
                    'staff_subject.*',
                    'subjects.subject_name as subjectname',
                    'subjects.subject_code as subjectcode',
                    'sub_category.category as category',
                    'users.name as name',
                    'users.lname as lname',
                    'courses.course_name as course',
                    'semesters.semester_name as sem'
                )->get();
                return $staffsubjects->map(function ($staffsubjects) {
                     return [
                $staffsubjects->name,
                $staffsubjects->course,
                $staffsubjects->sem,
                $staffsubjects->category,
                $staffsubjects->subjectcode,
                $staffsubjects->subjectname,
                
                
            ];
        });
       
    }
     public function headings(): array
{
    return [
        'Staff Name',     // Maps to $staffsubjects->name
        'Program',        // Maps to $staffsubjects->course
        'Semester',       // Maps to $staffsubjects->sem
        'Category',       // Maps to $staffsubjects->category
        'Course Code',    // Maps to $staffsubjects->subjectcode
        'Course Name',    // Maps to $staffsubjects->subjectname
    ];
}
    public function title(): string
    {
        if ($this->utype == 17) {
        $departmentn = Department::find($this->dept_id); // Use find() to avoid unnecessary query overhead
        if ($departmentn) {
            return 'Staff Course - ' . $departmentn->dep_name; // Use a separator for clarity
        } else {
            return 'Staff Course - Unknown Department'; // Fallback if department is not found
        }
    } else {
        return 'Staff Course';
    }
        
    }
     public function styles(Worksheet $sheet)
    {
        $sheet->getColumnDimension('A')->setWidth(20);
        $sheet->getColumnDimension('B')->setWidth(20);
        
        
        for ($column = ord('C'); $column <= ord('E'); $column++) {
            $columnLetter = chr($column);
            $sheet->getColumnDimension($columnLetter)->setWidth(25);
        }
         $sheet->getColumnDimension('F')->setWidth(70);   
        $sheet->getStyle('A1:F1')->applyFromArray([
            'font' => [
                'bold' => true,
                'size' => 12,
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
            ],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => [
                    'rgb' => 'FFFF00',
                ]
            ]
        ]);
            
        return [];
    }
    public function registerEvents(): array
    {
         $stfnme = 'Staff Course Report';
        if ($this->utype == 17) {
            $departmentn = Department::find($this->dept_id);
            if ($departmentn) {
                $stfnme = 'Staff Course Report - Department :' . $departmentn->dept_name;
            } else {
                $stfnme = 'Staff Course Report';
            }
        }
        return [
            AfterSheet::class => function (AfterSheet $event) use ($stfnme) {
                $sheet = $event->sheet;

                // Insert the title row before the column headings (row 1)
                $sheet->insertNewRowBefore(1, 1); // Insert a new row at position 1 for the title
                $sheet->setCellValue('A1', $stfnme);
                $sheet->mergeCells('A1:F1');
                $sheet->getStyle('A1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 14,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ]
                ]);
                
                // Add closing balance row after the last data entry
                $lastRow = $sheet->getHighestRow(); // Get the last row after data
                $sheet->getStyle('A2:F'.$lastRow)->applyFromArray([
                    'font' => [
                        'name' => 'Calibri',
                        'size' => 12,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '000000'],
                        ],
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                ]);
                
                $sheet->getRowDimension(1)->setRowHeight(25);
                $sheet->getRowDimension(2)->setRowHeight(20);
            }
        ];
    }
}
