<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use Illuminate\Support\Facades\Log;

use App\Models\Group;

class Groups extends Controller
{
    public function index()
    {
        try {
            $groups = Group::all();
            return view('group.report', compact('groups'));
        } catch (\Exception $e) {
            Log::error('Failed to fetch Groups: ' . $e->getMessage());
            return back()->withErrors('Failed to load Groups.');
        }
    }
    
    public function save(Request $request)
    {
        $validated = $request->validate([
                'name' => 'required|unique:groups,name'
            ]);
        
        try {
            $saved = Group::create($validated);
            if($saved){
                $msg_array = array('message'=>'Group Created Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Group Could not be Created !','alert-type'=>'error');
            }
        } catch (\Exception $e) {
            Log::error('Group creation failed: ' . $e->getMessage());
            $msg_array = array('message'=>'Group Could not be Created !','alert-type'=>'error');
        }
            
        return redirect()->route('group')->with($msg_array);
    }
    
    public function edit($gid)
    {
        try {
            $group = Group::findOrFail(decrypt($gid));
            return view('group.edit', compact('group'));
        } catch (ModelNotFoundException $e) {
            return redirect()->route('group')->withErrors('group not found.');
        } catch (\Exception $e) {
            Log::error('Error loading Group for edit: ' . $e->getMessage());
            return back()->withErrors('Failed to load the group for editing.');
        }
    }
    
    public function update(Request $request, $id)
    {
        $gid = decrypt($id);
        try {
            $validated = $request->validate([
                'name' => 'required|unique:groups,name,'.$gid.',id'
            ]);
                
            $group = Group::findOrFail($gid);
            $group->update($validated);
            
            $msg_array = array('message'=>'Group Updated Successfully !','alert-type'=>'success');
        } catch (\Exception $e) {
            Log::error('Group update failed: '.$e->getMessage());
            $msg_array = array('message'=>'Group Could not be Updated !','alert-type'=>'error');
        }
        
        return redirect()->route('group')->with($msg_array);
    }
    
    public function delete($id)
    {
        $gid = decrypt($id);
        try {
            $group = Group::findOrFail($gid);
            
            if($group->courses->count() ==0){
                $group->delete();
                $msg_array = array('message'=>'Group Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Group Could not be Deleted !','alert-type'=>'error');
            }
        } catch (\Exception $e) {
            Log::error('Group deletion failed: ' . $e->getMessage());
            $msg_array = array('message'=>'Group Could not be Deleted !','alert-type'=>'error');
        }
        
        return redirect()->route('group')->with($msg_array);
    }
}
