<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use Illuminate\Support\Facades\Log;

use App\Models\Course;
use App\Models\Semester;
use App\Models\Electives;
use App\Models\Student;
use App\Models\Department;
use App\Models\Group;
use App\Models\Department_Course;
use App\Models\Specialisation;

class Courses extends Controller
{
    public function add()
    {
        try {
            $departments = Department::all();
            $groups = Group::all();
            return view('course.add',compact('departments', 'groups'));
        } catch (\Exception $e) {
            Log::error('Failed to fetch Programmes: ' . $e->getMessage());
            return back()->withErrors('Failed to load Programmes.');
        }
    }
    
    public function save(Request $request)
    {
        $validated = $request->validate([
            'course_name' => 'required|unique:courses,course_name',
            'course_code' => 'required|unique:courses,course_code',
            'department_id' => 'required',
            'group_id' => 'nullable',
            'category_id'=>'required',
        ]);
        
        try {
            $i=0;
            $category = null;
            if($validated['category_id']=='UG')
            {
                $category=0;
            }
            elseif($validated['category_id']=='PG')
            {
               $category=1; 
            }
            $course = Course::create([
                'department_id' => $validated['department_id'],
                'group_id' => $validated['group_id'],
                'course_name' => $validated['course_name'],
                'course_code' => $validated['course_code'],
                'category'=>$category
            ]);
            if($course){
                $departmentcrs = Department_Course::create([
                    'dept_id' => $request->department_id,
                    'course_id' => $course->course_id,
                ]); 
                $i++; 
            }
            if($i > 0){
                $msg_array = array('message'=>'Programme Added Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Programme Could not be Added !','alert-type'=>'error');
            }
        } catch (\Exception $e) {
            Log::error('Programme creation failed: ' . $e->getMessage());
            $msg_array = array('message'=>'Programme Could not be Created !','alert-type'=>'error');
        }
        
        return redirect()->route('course.add')->with($msg_array);
    }
    
    public function report()
    {
        try {
            $courses = Course::all();
            return view('course.report', compact('courses'));
        } catch (\Exception $e) {
            Log::error('Failed to fetch Programmes: ' . $e->getMessage());
            return back()->withErrors('Failed to load Programmes.');
        }
    }
    
    public function edit($course_id)
    {
        try {
            $departments = Department::all();
            $groups = Group::all();
            $course = Course::findOrFail(decrypt($course_id));
            return view('course.edit', compact('departments', 'groups', 'course'));
        } catch (ModelNotFoundException $e) {
            return redirect()->route('course')->withErrors('programme not found.');
        } catch (\Exception $e) {
            Log::error('Error loading Programme for edit: ' . $e->getMessage());
            return back()->withErrors('Failed to load the programme for editing.');
        }
    }
    
    public function update(Request $request, $id)
    {
        $cid = decrypt($id);
        $validated = $request->validate([
            'course_name' => 'required|unique:courses,course_name,'.$cid.',course_id',
            'course_code' => 'required',
            'department_id' => 'required',
            'group_id' => 'nullable'
        ]);
        
        try {
            $course = Course::findOrFail($cid);
            $course->update($validated);
            
            $msg_array = array('message'=>'Programme Updated Successfully !','alert-type'=>'success');
        } catch (\Exception $e) {
            Log::error('Programme update failed: '.$e->getMessage());
            $msg_array = array('message'=>'Programme Could not be Updated !','alert-type'=>'error');
        }
        
        return redirect()->route('course')->with($msg_array);
    }
    
    public function delete($id)
    {
        $cid = decrypt($id);
        try {
            $course = Course::findOrFail($cid);
            if($course->semesters->count() == 0){
                $course->delete();
                $msg_array = array('message'=>'Programme Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Programme Could not be Deleted !','alert-type'=>'error');
            }
        } catch (\Exception $e) {
            Log::error('Programme deletion failed: ' . $e->getMessage());
            $msg_array = array('message'=>'Programme Could not be Deleted !','alert-type'=>'error');
        }
        
        return redirect()->route('course')->with($msg_array);
    }
    
    public function specialisation()
    {
        $courses=Course::all();
        $special = Specialisation::all();
        
        return view('course.special',compact('special','courses'));
    }
    
    public function specialisation_save(Request $request)
    {
        $validated = $request->validate([
            'specialisation' => 'required|unique:course_specialisation,specialisation',
        ]);
        
        $special = Specialisation::create([
            'course_id' => $request->course_id,
            'specialisation' => $request->specialisation,
        ]);
        
        if($special){
            $msg_array = array('message'=>' Added Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>' Could not be Added !','alert-type'=>'error');
        }
        return redirect()->route('specialisation')->with($msg_array);
    }

     public function specialisation_edit($sid){
        $courses=Course::all();
        $special = Specialisation::find(decrypt($sid));
        
        return view('course.specialedit',compact(array('special','courses')));
    }
   
    
    
       public function specialisation_update(Request $request){
        
        $specialtn_id = decrypt($request->specialtn_id);
        $validated = $request->validate([
            'specialisation' => 'required|unique:course_specialisation,specialisation,'.$specialtn_id.',specialtn_id',
        ]);
        
        $special = Specialisation::find($specialtn_id);
        $special->update([
            'course_id' => $request->course_id,
            'specialisation' => $request->specialisation,
        ]);
        
        if($special){
            $msg_array = array('message'=>' Updated Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>' Could not be Updated !','alert-type'=>'error');
        }
        return redirect()->route('specialisation')->with($msg_array);
    }
 
        public function specialisation_delete($sid){
        
        $special = Specialisation::find(decrypt($sid));
        
 if($special){
            if($special->count() > 0){
                $special->delete();
                $msg_array = array('message'=>'Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>' Could not be Deleted !','alert-type'=>'error');
            }
        }
        
        return redirect()->route('specialisation')->with($msg_array);
    }         
         

       public function addelectives(){
        
        $courses = Course::all();
        $sem=Semester::all();
        
        return view('course.addelectives',compact('courses','sem'));
    }
 
 public function get_elective(Request $request)
 {
    $crs_det = Course::find($request->cid);
    $sem_det = Semester::find($request->sid);
    $specialisation=Specialisation::where('course_id',$request->cid)->get();
    $elective=Electives::all();
    $student=Student::where('course_id',$request->cid)->orWhere('semester_id',$request->sid)->get();
//   echo $student;
    return view('course.addelectivesget',compact('crs_det','sem_det','specialisation','student'));
 }
    
    
public function addelectives_save(Request $request)
{
           $validated = $request->validate([
            'course_id' =>'required',
            'semester_id'=>'required',
        ]);
       $course_id=$request->course_id;
       $semester_id=$request->semester_id;
       
       $i=0;
       foreach($request->student_id as $stdnt)
       {
      
       
       
           $specialisation=$request->post("specialtn_id$stdnt");
           if($specialisation!=NULL)
           {
               
           
           $check=Electives::where('course_id',$course_id)->where('semester_id',$semester_id)->where('student_id',$stdnt)->first();
          if(!$check)
          {
              $addelectives=Electives::create([
                  'course_id'=>$course_id,
                  'semester_id'=>$semester_id,
                  'student_id'=>$stdnt,
                  'specialtn_id'=>$specialisation,
                  ]);
                  if($stdnt){ $i++; }
          }
          
           else
                {
                    $check->update(['specialtn_id' => $specialisation]);
                    $i++;
                }
           }
       }
       
      if($i>0){
            $msg_array = array('message'=>'Electives Added Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Electives Could not be Added !','alert-type'=>'error');
        }  
       return redirect()->route('addelectives')->with($msg_array);

}
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
}
