<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Student;
use App\Models\Acyear;
use App\Models\Promote;
use App\Models\Fee;
use App\Models\User;
use App\Models\StudentOtherFees;
use App\Models\Hostel;
use App\Models\HostelMonth;
use App\Models\HostelFee;
use App\Models\HostelPayment;
use App\Models\Course;
use App\Models\TodoModel;
use App\Models\Birthdaywish;
use Illuminate\Support\Facades\Auth;
use App\Models\AcademicCalendar;

use DB; 
use Carbon\Carbon;
class Dashboard extends Controller
{
    public function counts(){
        $acyearId = getAcyear();
        $students = Promote::where('ac_id', $acyearId)->distinct('student_id')->count('student_id');
        $admissions = Student::where('ac_id', $acyearId)->count();
        $staffs = User::count();
        
        $data = array('0'=>$students,'1'=>$admissions,'2'=>$staffs);
        echo json_encode($data);  
    }
    
    public function fee_counts(){
        $curdate = date('Y-m-d');
        $start = date('Y-08-01');
        $end = date('Y-08-t');
        $fees = Fee::whereNull('deleted_at')->select(DB::raw("SUM(CASE WHEN paid_date='$curdate' THEN fee_amount+fine_amount-discount ELSE 0 END) as daily"),DB::raw("SUM(CASE WHEN paid_date between '$start' and '$end' THEN fee_amount+fine_amount-discount ELSE 0 END) as monthly"))->first();
        $daily = '&#8377 '.$fees->daily;
        
        $currentDate = Carbon::today();
        $fees = Fee::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->sum('fee_amount');
        $discount= Fee::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->sum('discount');
        $daily_fine=Fee::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->sum('fine_amount');
        $total_fee =$fees-$discount+$daily_fine;
        
        $month_start = date('Y-m-01');
        $month_end = date('Y-m-t');
        $monthlyfee=Fee::whereNull('deleted_at')->whereBetween('paid_date',[$month_start,$month_end])->sum('fee_amount');
        $monthly_discount=Fee::whereNull('deleted_at')->whereBetween('paid_date',[$month_start,$month_end])->sum('discount');
        $monthly_fine=Fee::whereNull('deleted_at')->whereBetween('paid_date',[$month_start,$month_end])->sum('fine_amount');
        $monthlyfeewithdis=$monthlyfee-$monthly_discount+$monthly_fine;
        // $monthly = '&#8377 '.$fees->monthly;
        // $data = array('0'=>$total_fee,'1'=>$monthly);
        
        $data = array('0'=>$total_fee,'1'=>$monthlyfeewithdis);
        echo json_encode($data);
    }
    
    public function other_counts(){
        $curdate = date('Y-m-d');
        $otherfee = StudentOtherFees::whereDate('date',$curdate)->sum('total_amount');
        $hostelfee = HostelPayment::whereDate('paid_date',$curdate)->where('status',1)->select(DB::raw("COALESCE(SUM(amount - discount), 0) as fees"))->first();
       
        $today = \Carbon\Carbon::now()->format('m-d');
        $birthdays = DB::table('users')
            ->whereRaw('DATE_FORMAT(dob,"%m-%d") = ?', [$today])
            ->select('name')
            ->get();
            $birthdaycount = $birthdays->count();
        
        $other = '&#8377 '.$otherfee;
        $hostel = '&#8377 '.$hostelfee->fees;
        
        $data = array('0'=>$other,'1'=>$hostel,'2'=>$birthdaycount);
        echo json_encode($data);
    }
    
    public function graph(){
        $acyears = Acyear::all();
        $years = [];
        $counts = [];
        foreach($acyears as $acyear){
            $students = Student::where('ac_id', $acyear->ac_id)->count();
            $years[] = date('Y', strtotime($acyear->sdate)).'-'.date('y', strtotime($acyear->edate));
            $counts[] = $students;
        }
        
        return view('dashboard.graph',compact('acyears','years','counts'));
    }
    
    public function graph_course(Request $request){
        $acyearid = $request->year;
        $courselist = Course::all();
        $courses = []; $counts = [];
        foreach($courselist as $course){
            $students = Student::where('ac_id',$acyearid)->where('course_id',$course->course_id)->count();
            $courses[] = $course->course_code;
            $counts[] = $students;
        }
        
        $data = array('0'=>$courses,'1'=>$counts);
        echo json_encode($data);
    }
 
    public function total_students()
    {
        $currentAcyear=Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $ac_id = $currentAcyear->ac_id;
        $totalstudent=Promote::where('ac_id',$ac_id)->distinct()->count('student_id');
        
        return view('dashboard.total_students',compact('totalstudent'));
    }

    public function get_total_students(Request $request)
    {
        $col_order = ['adno','rollno','name','fname','mname','mobile','course_name','semester_name'];
        $total_data = Student::whereNull('deleted_at')->count();
        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $col_order[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $current_academic_year_id = $cur_acyear->ac_id;
        
        $query = Student::join(DB::raw("(SELECT * FROM promotes WHERE (student_id, promote_id) IN (
                    SELECT student_id, MAX(promote_id) AS promote_id 
                    FROM promotes 
                    GROUP BY student_id)) as latest_promotes"), 
                        function($join) {
                            $join->on('students.student_id', '=', 'latest_promotes.student_id');
                        })
                    ->join('courses', 'latest_promotes.course_id', '=', 'courses.course_id')
                    ->join('semesters', 'latest_promotes.semester_id', '=', 'semesters.semester_id')
                    ->where('latest_promotes.ac_id', $current_academic_year_id)
                    ->where('students.tcstatus', '0');
                    
        if(!empty($request->input('search.value'))){
            $search = $request->input('search.value');
            $query->where(function ($qry) use ($search){
                $qry->where('adno','like','%'.$search.'%')->
                    orWhere('rollno','like','%'.$search.'%')->
                    orWhere('name','like','%'.$search.'%')->
                    orWhere('mobile','like','%'.$search.'%')->
                    orWhere('course_name','like','%'.$search.'%')->
                    orWhere('semester_name','like','%'.$search.'%');
            });
        }
        
        $total_filtered = $query->distinct('students.student_id')->count();
        
        $students  =   $query->select('students.*','courses.course_name','semesters.semester_name')->distinct('students.student_id')->offset($start)->limit($limit)->orderBy($order,$dir)->get();
        
        $data = array();
        if($students){
            foreach($students as $student){ 
                $row['adno'] = $student->adno;
                $row['rollno'] = $student->rollno;
                $row['name'] = '<a href="'.route('searchstudent.get.view', encrypt($student->student_id)).'" title="View" target="_blank"><u>' . wordwrap(strtoupper($student->name), 20, "<br />\n") . '</u></a>';
                $row['fathername'] = wordwrap(strtoupper($student->fname), 20, "<br />\n");
                $row['mothername'] = wordwrap(strtoupper($student->mname), 20, "<br />\n");
                $row['mobile'] = $student->mobile;
                $row['course'] = wordwrap(strtoupper($student->course_name), 20, "<br />\n");
                $row['semester'] = strtoupper($student->semester_name);
                $html="";
                $row['action'] = $html;
                
                $data[] = $row;
            }
        }
        
        $json = array(
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data
        );
            
        echo json_encode($json);
    }

    public function total_newadmission()
    {
        return view('dashboard.total_newadmission');
    }

    public function get_newadmission(Request $request)
    {
        $col_order = ['adno','rollno','name','fname','mname','mobile','course_name'];
        $total_data = Student::whereNull('deleted_at')->count();
        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $col_order[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $current_academic_year_id = $cur_acyear->ac_id;
        $query  =   Student::join('courses','students.course_id','=','courses.course_id')->
                    where('students.ac_id',$current_academic_year_id);
        if(!empty($request->input('search.value'))){
            $search = $request->input('search.value');
            $query->where(function ($qry) use ($search){
                $qry->where('adno','like','%'.$search.'%')->
                    orWhere('name','like','%'.$search.'%')->
                    orWhere('mobile','like','%'.$search.'%')->
                    orWhere('course_name','like','%'.$search.'%');
            });
        }
        
        $total_filtered = $query->count();
        $students = $query->select('students.*','courses.course_name')->offset($start)->limit($limit)->orderBy($order,$dir)->get();
        
        $data = array();
        if($students){
            foreach($students as $student){
                $row['adno'] = $student->adno;
                $row['rollno'] = $student->rollno;
                $row['name'] = '<a href="'.route('searchstudent.get.view', encrypt($student->student_id)).'" title="View" target="_blank"><u>' . wordwrap(strtoupper($student->name), 20, "<br />\n") . '</u></a>';
                $row['fathername'] = wordwrap(strtoupper($student->fname), 20, "<br />\n");
                $row['mothername'] = wordwrap(strtoupper($student->mname), 20, "<br />\n");
                $row['mobile'] = $student->mobile;
                $row['course'] = wordwrap(strtoupper($student->course_name), 20, "<br />\n");
                $html="";
                $row['action'] = $html;
                
                $data[] = $row;
            }
        }
        
        $json = array(
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data
        );
            
        echo json_encode($json);
    }
    
    public function total_staff()
    {
        return view('dashboard.total_staff');
    }
    
    public function get_staff(Request $request)
    {
        $col_order = ['employee_id_num','name','type_name','mobile','father_guardian','address'];
        $total_data = Student::whereNull('deleted_at')->count();
        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $col_order[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $current_academic_year_id = $cur_acyear->ac_id;
        $query  =   User::join('user_types','user_types.type_id','=','users.type_id')->where('users.status','1');
        if(!empty($request->input('search.value'))){
            $search = $request->input('search.value');
            $query->where(function ($qry) use ($search){
                $qry->where('employee_id_num','like','%'.$search.'%')->
                    orWhere('name','like','%'.$search.'%')->
                    orWhere('mobile','like','%'.$search.'%');
            });
        }
        
        $total_filtered = $query->count();
        $students = $query->select('users.*','user_types.type_name')->offset($start)->limit($limit)->orderBy($order,$dir)->get();
        
        $data = array();
        if($students){
            foreach($students as $student){
                $row['employeeid'] = $student->employee_id_num;
                $row['name'] = '<a href="'.route('user.edit', encrypt($student->user_id)).'" title="Edit" target="_blank"><u>' . wordwrap(strtoupper($student->name), 20, "<br />\n") . '</u></a>';
                $row['designation'] = wordwrap(strtoupper($student->type_name), 20, "<br />\n");
                $row['mobile'] = $student->mobile;
                $row['father'] = wordwrap(strtoupper($student->father_guardian), 20, "<br />\n");
                $row['address'] = wordwrap(strtoupper($student->address), 20, "<br />\n");
                $html="";
                $row['action'] = $html;
                
                $data[] = $row;
            }
        }
        
        $json = array(
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data
        );
            
        echo json_encode($json);
    }
    
    public function monthlyfee(Request $request)
    {
        $fees=array();
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $ac_id=$cur_acyear->ac_id;
        $month_start = date('Y-m-01');
        $month_end = date('Y-m-t');
        $query = Fee::join('students','fees.student_id','=','students.student_id')->
                    join('courses','students.course_id','=','courses.course_id')->
                    join('semesters','fees.semester_id','=','semesters.semester_id')->
                    where('fees.status',1)->
                    where('fees.paid_date','>=',$month_start)->
                    where('fees.paid_date','<=',$month_end);
                    
        $results = $query->select('fees.receipt_no','fees.paid_date')->groupBy(['fees.receipt_no','fees.paid_date'])->get();
        foreach($results as $result){
            $fee_det = Fee::join('students','fees.student_id','=','students.student_id')->
                        join('courses','students.course_id','=','courses.course_id')->
                        join('semesters','fees.semester_id','=','semesters.semester_id')->
                        where('fees.status',1)->
                        where('fees.receipt_no',$result->receipt_no)->
                        where('fees.paid_date',$result->paid_date)->
                        select('fees.receipt_no','fees.paid_date','students.adno','students.name',DB::raw('sum(fee_amount) as fee'),DB::raw('sum(fine_amount) as fine'),DB::raw('sum(discount) as discount'))->get()->first();
                       
            $fees[] = array('recno'=>$fee_det->receipt_no,'pdate'=>$fee_det->paid_date,'adno'=>$fee_det->adno,'name'=>$fee_det->name,'fee'=>$fee_det->fee,'fine'=>$fee_det->fine,'discount'=>$fee_det->discount);
        }
        
        return view('dashboard.monthlyfee',compact('month_start','month_end','fees'));
    }
    
   public function daily_fee(Request $request)
     {
       
        $fees=array();
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $ac_id=$cur_acyear->ac_id;
        $current_date = date('Y-m-d');
        $query = Fee::join('students','fees.student_id','=','students.student_id')->
                    join('courses','students.course_id','=','courses.course_id')->
                    join('semesters','fees.semester_id','=','semesters.semester_id')->
                    where('fees.status',1)->
                    where('fees.paid_date',$current_date);
                   
        
        $results = $query->select('fees.receipt_no','fees.paid_date')->groupBy(['fees.receipt_no','fees.paid_date'])->get();
        foreach($results as $result){
            $fee_det = Fee::join('students','fees.student_id','=','students.student_id')->
                        join('courses','students.course_id','=','courses.course_id')->
                        join('semesters','fees.semester_id','=','semesters.semester_id')->
                        where('fees.status',1)->
                        where('fees.receipt_no',$result->receipt_no)->
                        where('fees.paid_date',$result->paid_date)->
                        select('fees.receipt_no','fees.paid_date','students.adno','students.name',DB::raw('sum(fee_amount) as fee'),DB::raw('sum(fine_amount) as fine'),DB::raw('sum(discount) as discount'))->get()->first();
                       
       $fees[] = array('recno'=>$fee_det->receipt_no,'pdate'=>$fee_det->paid_date,'adno'=>$fee_det->adno,'name'=>$fee_det->name,'fee'=>$fee_det->fee,'fine'=>$fee_det->fine,'discount'=>$fee_det->discount);
        
        }
        
        return view('dashboard.daily_fee',compact('current_date','fees'));

  }

public function dueamount()
{
    
   return view('dashboard.dueamount'); 
    
}
 public function dueamount_get(Request $request)
 {
       
        $col_order = ['adno','name','course_name','semester_name','',''];
        $total_data = Student::whereNull('deleted_at')->count();
        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $col_order[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        $current_date = date('Y-m-d');
       
        
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        
        $query  =   Student::join('promotes','students.student_id','=','promotes.student_id')->
                        join('courses','promotes.course_id','=','courses.course_id')->
                        join('semesters','promotes.semester_id','=','semesters.semester_id')->
                        where('promotes.ac_id',$cur_acyear->ac_id);
        if(!empty($request->input('search.value'))){
            $this->search = $request->input('search.value');
            $query->where(function($qry) {
                return $qry->where('adno','like','%'.$this->search.'%')->
                orWhere('name','like','%'.$this->search.'%')->
                orWhere('course_name','like','%'.$this->search.'%')->
                orWhere('semester_name','like','%'.$this->search.'%');
            });
        }
        $total_filtered = $query->count();
        $students  =   $query->select('students.*','courses.course_name','semesters.semester_name')->
                    offset($start)->limit($limit)->groupBy('promotes.student_id')->orderBy($order,$dir)->get();
                 
        $data = array();
        if($students){
            foreach($students as $student){
                $row['adno'] = $student->adno;
                $row['name'] = $student->name;
                $row['course'] = $student->course_name;
                $row['semester'] = $student->semester_name;
                
                $fees = Fee::studentFees($student->student_id,$current_date);
                $famount=0;
                foreach($fees as $ac_id => $sem_details){
                    foreach($sem_details as $sem_id => $term_details){
                    foreach($term_details['fee_structure'] as $term_id => $fee_details){
                        foreach($fee_details['fee_type'] as $ftype_id => $ftype_details){
                            $famount += $ftype_details['due_amount']+$ftype_details['fine_amount'];
                        }
                    }
                }
                }
                $row['due'] = number_format($famount,2);
                $html="";
                $row['action'] = $html;
                
                $data[] = $row;
            }
        }
        
        $json = array(
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data
        );
            
        echo json_encode($json);
 }

// public function dueamount_total(Request $request)
// {
//         $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
//         $current_date = date('Y-m-d');
//         $query  =   Student::join('promotes','students.student_id','=','promotes.student_id')->
//                         join('courses','promotes.course_id','=','courses.course_id')->
//                         join('semesters','promotes.semester_id','=','semesters.semester_id')->
//                         where('promotes.ac_id',$cur_acyear->ac_id);
                        
//         $total=0;
        
//         $students  =  $query->get();
//         if($students){
//             foreach($students as $student){
//                 $fees = Fee::studentFees($student->student_id,$cur_acyear->ac_id);
               
//                 $famount=0;
//                 foreach($fees as  $ac_id => $sem_details){
//                     foreach($sem_details as $sem_id => $term_details){
//                     foreach($term_details['fee_structure'] as $term_id => $fee_details){
//                         foreach($fee_details['fee_type'] as $ftype_id => $ftype_details){
//                             $famount += $ftype_details['due_amount']+$ftype_details['fine_amount'];
//                         }
//                     }
//                 }
//                 }
//                 $total +=$famount;
//                 console.log($total);
                
            
//         }
//         }
//         echo number_format($total,2);
// }

   public function dailyotherfee(Request $request)
   {
       $current_date = date('Y-m-d');
       $otherfees=StudentOtherFees::where('date',$current_date)->get(); 
       return view('dashboard.dailyotherfee',compact('current_date','otherfees'));
       
   }
    
public function dailyhostel()
{
    return view('dashboard.dailyhostel');
}

public function dailyhostel_get(Request $request)
{
   
        $col_order = ['receipt_no','paid_date','adno','name','course_name','semester_name'];
        $total_data = HostelPayment::whereNull('deleted_at')->groupBy('receipt_no')->count();
        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $col_order[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        $current_date = date('Y-m-d');

        
        
        $query = HostelPayment::join('students','hostel_payments.student_id','=','students.student_id')->
                    join('courses','students.course_id','=','courses.course_id')->
                    join('semesters','hostel_payments.semester_id','=','semesters.semester_id')->
                    join('users','hostel_payments.created_by','=','users.user_id')->
                    where('hostel_payments.status',1)->where('hostel_payments.paid_date',$current_date)->
                    whereNull('hostel_payments.deleted_at');
       
        
        if(!empty($request->input('search.value'))){
            $this->search = $request->input('search.value');
            $query->where(function($qry) {
                return $qry->orWhere('hostel_payments.receipt_no','like','%'.$this->search.'%')->
                orWhere('students.adno','like','%'.$this->search.'%')->
                orWhere('students.name','like','%'.$this->search.'%')->
                orWhere('courses.course_name','like','%'.$this->search.'%')->
                orWhere('semesters.semester_name','like','%'.$this->search.'%');
               
            });
        };
        $total_filtered = $query->groupBy('hostel_payments.receipt_no')->count();
        $fees  =   $query->select('hostel_payments.receipt_no','hostel_payments.paid_date','students.adno','students.name',DB::raw('sum(hostel_payments.amount) as amt'),DB::raw('sum(hostel_payments.discount) as disc'),'courses.course_name','semesters.semester_name')->
                    offset($start)->limit($limit)->groupBy('hostel_payments.receipt_no')->orderBy($order,$dir)->get();
                 
        $data = array();
        if($fees){
            foreach($fees as $fee){
                $row['recno'] = $fee->receipt_no;
                $row['date'] = date('d/m/Y',strtotime($fee->paid_date));
                $row['adno'] = $fee->adno;
                $row['name'] = $fee->name;
                $row['course'] = $fee->course_name;
                $row['semester'] = $fee->semester_name;
                $row['amount'] = number_format(($fee->amt-$fee->disc),2);
                $html="";
                $row['action'] = $html;
                
                $data[] = $row;
            }
        }
        
        $json = array(
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data
        );
            
        echo json_encode($json);
}

 public function dailyhostel_total(Request $request)  
 {
    
        $amount=0;
        $current_date = date('Y-m-d');
        $query = HostelPayment::join('students','hostel_payments.student_id','=','students.student_id')->
                    join('courses','students.course_id','=','courses.course_id')->
                    join('semesters','hostel_payments.semester_id','=','semesters.semester_id')->
                    where('hostel_payments.status',1)->where('hostel_payments.paid_date',$current_date)->
                    whereNull('hostel_payments.deleted_at');
                   
                    
                    
        $data = $query->select(DB::raw("sum(hostel_payments.amount-hostel_payments.discount) as fees"))->get()->first();
        if(!empty($data)){
            $amount = $data->fees;
        }
        
        
        echo number_format($amount,2); 
 }  
    
public function dueamount_total(Request $request)
{
      
        
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $cdate = date('Y-m-d');
        $query  =   Student::join('promotes','students.student_id','=','promotes.student_id')->
                        join('courses','promotes.course_id','=','courses.course_id')->
                        join('semesters','promotes.semester_id','=','semesters.semester_id')->
                        where('promotes.ac_id',$cur_acyear->ac_id);
        
        $total=0;
        $students  =   $query->get();
        if($students){
            foreach($students as $student){
                $fees = Fee::studentFees($student->student_id,$cdate);
                $famount=0;
                foreach($fees as $ac_id => $sem_details){
                    foreach($sem_details as $sem_id => $term_details){
                    foreach($term_details['fee_structure'] as $term_id => $fee_details){
                        foreach($fee_details['fee_type'] as $ftype_id => $ftype_details){
                            $famount += $ftype_details['due_amount']+$ftype_details['fine_amount'];
                             
                           
                        }
                    }
                }
                }
               $total +=$famount;
            }
        }
        echo number_format($total,2);
}
    
    
    
    
    
    
// ..........new............    
    
    
    public function students()
    {
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $ac_id = $cur_acyear->ac_id;
        $promotes=Promote::where('ac_id',$ac_id)->count('student_id');
        return view('dashboard.students_new',compact('promotes'));
    }
    
    public function newadmission()
    {
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $ac_id = $cur_acyear->ac_id;
        $newadmission=Student::where('ac_id',$ac_id)->count('student_id');
      
        return view('dashboard.newadmission_new',compact('newadmission'));
    }
    
    
   public function newstaff()
   {
        $totalstaff=User::count('user_id');
         return view('dashboard.newstaff',compact('totalstaff'));
   }
    
    public function monthlyfee_new()
    {
        $month_start = date('Y-m-01');
        $month_end = date('Y-m-t');
        $monthlyfee=Fee::whereNull('deleted_at')->whereBetween('paid_date',[$month_start,$month_end])->sum('fee_amount');
        
        $monthly_discount=Fee::whereNull('deleted_at')->whereBetween('paid_date',[$month_start,$month_end])->sum('discount');
        $monthly_fine=Fee::whereNull('deleted_at')->whereBetween('paid_date',[$month_start,$month_end])->sum('fine_amount');
        $monthlyfeewithdis=$monthlyfee-$monthly_discount+$monthly_fine;
        
        return view('dashboard.monthlyfee_new',compact('monthlyfeewithdis'));
        
    }
    
    public function dailyfeenew()
    {
         $currentDate = Carbon::today();
         $fees = Fee::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->sum('fee_amount');
         $discount= Fee::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->sum('discount');
         $daily_fine=Fee::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->sum('fine_amount');
         $total_fee =$fees-$discount+$daily_fine;
      
        
         return view('dashboard.dailyfeenew',compact('total_fee'));
    }
    
    
    public function totalduenew()
    {
         $currentDate = Carbon::today();
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $studentArray = Student::whereNull('deleted_at')->get();
        $studentID = [];

        foreach ($studentArray as $student) {
        $studentID[] = $student->student_id;
        }
          $dueAmount = [];
    if ($cur_acyear) {
   
    $ac_id = $cur_acyear->ac_id;
    
    $dueAmount = [];

   
    foreach ($studentID as $student_id) {
     
        $dueAmount[$student_id] = Fee::studentFees($student_id, $ac_id, $currentDate);
      
    }

 
} else {
 
  
    throw new Exception('No default academic year found.');
}
    return view('dashboard.totalduenew',compact('dueAmount'));

    }
    
   public function otherfeenew() 
   {
        $currentDate = Carbon::today();
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $ac_id = $cur_acyear->ac_id;
        $totalAmount = StudentOtherFees::whereDate('created_at', $currentDate)->where('ac_id',$ac_id)->sum('total_amount'); 
        return view('dashboard.otherfeenew',compact('totalAmount'));
   }
    
   public function hostelfeenew()
   {
         $currentDate = Carbon::today();
         $hostels = HostelPayment::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->select(DB::raw("sum(amount-discount) as fees"))->get()->first();
        return view('dashboard.hostelfeenew',compact('hostels'));
         
   }
    public function acyeargraph()
    {
        $allacyearstudent=Student::all();
        $allAcademicYears = Acyear::whereNull('deleted_at')->get();
        $admissionsByYear = [];
        $admissionsByCourseAndYear = [];
        foreach ($allAcademicYears as $academicYear) {
        $startYear = date('Y', strtotime($academicYear->sdate));
        $endYear = date('Y', strtotime($academicYear->edate));
        $formattedYearRange = $startYear . '-' . $endYear;
        $admissionsCount = Student::where('ac_id', $academicYear->ac_id)->count('student_id');
        $admissionsByYear[$formattedYearRange] = $admissionsCount; 
        $admissionsByCourse = [];
        $courses = Course::whereNull('deleted_at')->get();
        foreach ($courses as $course) {
            $courseId = $course->course_id;
            $course_name=$course->course_code;
            $admissionsCount = Student::where('ac_id', $academicYear->ac_id)
                                      ->where('course_id', $courseId)
                                      ->whereNull('deleted_at')
                                      ->count();
            $admissionsByCourse[$course_name] = $admissionsCount;
            // print_r($admissionsByCourse);
         
        }
        $admissionsByCourseAndYear[$formattedYearRange] = $admissionsByCourse;
    //   print_r($admissionsByCourseAndYear);
     
       
        }
        return view('dashboard.acyeargraph',compact('admissionsByYear','admissionsByCourseAndYear','courses'));
        
    }
   
    public function total_cards()
    {
          
        
        $month_start = date('Y-m-01');
        $month_end = date('Y-m-t');
        $currentDate = Carbon::today();
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        $ac_id = $cur_acyear->ac_id;
       
        $promotes=Promote::where('ac_id',$ac_id)->count('student_id');
        $totalAmount = StudentOtherFees::whereDate('created_at', $currentDate)->where('ac_id',$ac_id)->sum('total_amount'); 
        $students = Student::whereNull('deleted_at')->whereBetween('adate', [$cur_acyear->sdate, $cur_acyear->edate])->count();
        $courses = Course::whereNull('deleted_at')->count();
        $fees = Fee::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->sum('fee_amount');
        $discount= Fee::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->sum('discount');
        $daily_fine=Fee::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->sum('fine_amount');
        $total_fee =$fees-$discount+$daily_fine;
       
        $hostels = HostelPayment::whereNull('deleted_at')->whereDate('paid_date',$currentDate)->select(DB::raw("sum(amount-discount) as fees"))->get()->first();
        $newadmission=Student::where('ac_id',$ac_id)->count('student_id');
        $totalstaff=User::count('user_id');
        $monthlyfee=Fee::whereNull('deleted_at')->whereBetween('paid_date',[$month_start,$month_end])->sum('fee_amount');
        
        $monthly_discount=Fee::whereNull('deleted_at')->whereBetween('paid_date',[$month_start,$month_end])->sum('discount');
        $monthly_fine=Fee::whereNull('deleted_at')->whereBetween('paid_date',[$month_start,$month_end])->sum('fine_amount');
        $monthlyfeewithdis=$monthlyfee-$monthly_discount+$monthly_fine;
        $allacyearstudent=Student::all();
        $allAcademicYears = Acyear::whereNull('deleted_at')->get();
        $admissionsByYear = [];
        $admissionsByCourseAndYear = [];
        foreach ($allAcademicYears as $academicYear) {
        $startYear = date('Y', strtotime($academicYear->sdate));
        $endYear = date('Y', strtotime($academicYear->edate));
        $formattedYearRange = $startYear . '-' . $endYear;
        $admissionsCount = Student::where('ac_id', $academicYear->ac_id)->count('student_id');
        $admissionsByYear[$formattedYearRange] = $admissionsCount; 
        $admissionsByCourse = [];
        $courses = Course::whereNull('deleted_at')->get();
        foreach ($courses as $course) {
            $courseId = $course->course_id;
            $course_name=$course->course_code;
            $admissionsCount = Student::where('ac_id', $academicYear->ac_id)
                                      ->where('course_id', $courseId)
                                      ->whereNull('deleted_at')
                                      ->count();
            $admissionsByCourse[$course_name] = $admissionsCount;
            // print_r($admissionsByCourse);
          
        }
        $admissionsByCourseAndYear[$formattedYearRange] = $admissionsByCourse;
    //   print_r($admissionsByCourseAndYear);
     
       
        }
        
        $studentArray = Student::whereNull('deleted_at')->get();
        $studentID = [];

        foreach ($studentArray as $student) {
        $studentID[] = $student->student_id;
        }

   
        $dueAmount = [];
if ($cur_acyear) {
   
    $ac_id = $cur_acyear->ac_id;
    
    $dueAmount = [];

   
    foreach ($studentID as $student_id) {
     
        $dueAmount[$student_id] = Fee::studentFees($student_id, $ac_id, $currentDate);
      
    }

 
} else {
 
  
    throw new Exception('No default academic year found.');
}
    //   foreach ($studentID as $student_id) {
    //   $dueAmount[$student_id] = Fee::studentFees($student_id, $currentDate);
       
    //     }

         $acyear=Acyear::all();
        return view('dashboard.dashboard_cards', compact('students', 'promotes', 'courses', 'fees', 'hostels', 'totalAmount', 'newadmission', 'totalstaff', 'monthlyfee', 'admissionsByYear', 'admissionsByCourseAndYear','dueAmount','total_fee','monthlyfeewithdis','acyear'));
    }
    
    // public function graph()
//     {
//         // $allacyearstudent=Student::all();
//         $allAcademicYears = Acyear::whereNull('deleted_at')->get()->toArray();
//           $student_count=[];
//           $academic_year_labels = [];
//       foreach ($allAcademicYears as $academicYear) {
//               $admissionsCount = Student::where('ac_id', $academicYear->ac_id)->count('student_id');
//               $student_count[]=$admissionsCount;
//               $academic_year_labels[] = $academicYear->sdate . ' - ' . $academicYear->edate; 

//       }
       
       
      
//   return view('dashboard.dashboard_graph',compact('student_count','allAcademicYears','academic_year_labels')); 
      
//     }
    public function agraph()
{
    $allAcademicYears = Acyear::whereNull('deleted_at')->get();
    $student_count = [];
    $academic_year_labels = [];

    foreach ($allAcademicYears as $academicYear) {
        $admissionsCount = Student::where('ac_id', $academicYear->ac_id)->count('student_id');
        $student_count[] = $admissionsCount;
        $startYear = date('Y', strtotime($academicYear->sdate));
        $endYear = date('Y', strtotime($academicYear->edate));
        $academic_year_labels[] = "{$startYear}-{$endYear}";
     
    }

   
    return view('dashboard.dashboard_graph',compact('student_count','academic_year_labels','allAcademicYears'));
        
    
}
public function getCourseData($ac_id)
{
     
     $courses = Course::all();
     $course_names = [];
     $student_counts = [];
     foreach ($courses as $course) {
            $student_count = Student::where('course_id', $course->course_id)->where('ac_id',$ac_id)->count('student_id');
            $course_names[] = $course->course_code; 
            $student_counts[] = $student_count;
        }
         return response()->json([
            'course_names' => $course_names,
            'student_counts' => $student_counts
        ]);
}

    
 public function datepicker()
 {
     return view('dashboard.datepicker');
 }


public function todo()
{
  
    return view('dashboard.todo');
}
    
 public function todo_save(Request $request)
 {
      $current_date = date('Y-m-d');
   
     $task=$request->input('task');
     $add=TodoModel::create([
         'task'=>$task,
         'date'=>$current_date,
         'status'=>'1',
         ]);
         return redirect()->route('home');
 }
    
    
public function todo_view()
{
      $todo=TodoModel::all();
      return view('dashboard.viewtodo',compact('todo'));
}

    
    public function todo_update($id)
   {
     
       try {
        $todo = TodoModel::findOrFail($id);
        $todo->status = 0;
        $todo->save();

        return true;
    } catch (\Exception $e) {
        return response()->json(['success' => false, 'error' => $e->getMessage()]);
    }
         $todo = TodoModel::find($request->todo_id);
    
    } 

public function  birthday()
{
    $today = \Carbon\Carbon::now()->format('m-d');
    $birthdays=User::join('deparment','deparment.id','=','users.dept_id')
    ->whereRaw('DATE_FORMAT(users.dob, "%m-%d") = ?', [$today])
    ->select('users.user_id','users.name','users.image','users.mobile','deparment.dept_name','users.dateof_join','users.email','users.address')
     ->get();
      foreach ($birthdays as $birthday) {
        $birthday->has_sent_wish = Birthdaywish::where('to', $birthday->user_id)
                                                 ->where('from', Auth::id())
                                                 ->exists();
    }

    
            return view('dashboard.birthday',compact('birthdays'));
}

    public function birthday_msg(Request $request)
    {
        $user_id=User::find($request->user_id);
       
        $userId = Auth::id();
        $curdate = date('Y-m-d');
        $message=$request->message;
       $exist=Birthdaywish::where('to',$user_id->user_id)->where('from',$userId)->exists();
       if(!$exist)
       {
            $wish=Birthdaywish::create([
            'to'=>$user_id->user_id,
            'from'=>$userId,
            'message'=>$message,
            'date'=>$curdate,
            
            ]);
       }
       
            return redirect()->route('dash.birthday')->with('message','Birthday wish sent successfully!');
        
    }
    
    
   public function acalendar()
   {
        $acyears = Acyear::all();
        $ac_id = getAcyear();
        $ac_events_all =AcademicCalendar::where('ac_id',$ac_id)->get();
        $events = $ac_events_all->map(function ($event) {
        $color = '#000000';
        if ($event->type == 1) {
            $color = '#00FF00';
        } elseif ($event->type == 2) {
            $color = '#FF0000'; 
        } elseif ($event->type == 3) {
            $color = '#FFC0CB'; 
        } elseif ($event->type == 4) {
            $color = '#90EE90'; 
        }     
        return [
            'title' => $event->event,
            'start' => \Carbon\Carbon::parse($event->ev_date)->toDateString(),
            'description' => $event->event,
            
            'color' => $color  
        ];
    });

    return response()->json($events);
        
    }    
}









