<?php

namespace App\Http\Controllers\Staff;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

use App\Models\Acyear;
use App\Models\ClassTeacher;
use App\Models\Promote;
use App\Models\StaffAttendance;
use App\Models\Periods;
use App\Models\FyugpTimetable;
use App\Models\FyugpTimetableInterchange;
use App\Models\Timetable;
use App\Models\TimetableInterchange;
use App\Models\Semester;
use App\Models\StaffSubject;
use App\Models\Group;

use DB;

class Ajax extends Controller
{
    public function tutor_semester($courseId){
        $acyearId = getAcyear();
        $userId = Auth::id();
        $semesters = [];
        $results = ClassTeacher::join('semesters', 'classteacher.semester_id', '=', 'semesters.semester_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->where('classteacher.course_id', $courseId)->select('semesters.*')->groupBy('classteacher.semester_id')->get();
        foreach($results as $result){
            $semesters[$result->semester_id] = $result->semester_name;
        }
        return json_encode($semesters);
    }
    
    public function tutor_students($courseId, $semesterId){
        $acyearId = getAcyear();
        $students = [];
        $results = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')->where('promotes.ac_id', $acyearId)->where('promotes.course_id', $courseId)->where('promotes.semester_id', $semesterId)->select('students.student_id','students.adno','students.name')->groupBy('promotes.student_id')->orderBy('students.name', 'ASC')->get();
        foreach($results as $result){
            $students[$result->student_id] = $result->adno.' - '.$result->name;
        }
        return json_encode($students);
    }
    
    public function attendance(){
        
        $userId = Auth::id();
        $attendance = StaffAttendance::where('user_id', $userId)
                    ->whereBetween('date',[date('Y-m-01'), date('Y-m-t')])
                    ->select(DB::raw("COUNT(*) AS total"), DB::raw("SUM(am+pm) AS present"))->first();
        $percentage=$total=$present=$absent=0;
        if($attendance && $attendance->total > 0){
            $total = $attendance->total;
            $present = $attendance->present*0.5;
            $absent = $total-$present;
            $percentage = round(($present/$total)*100);
        }
        
        return view('staff.dashboard.attendance',compact('total', 'present', 'absent', 'percentage'));
    }
    
    public function timetable(){
        
        $userId = Auth::id();
        $day = date('l');
        $acyearId = getAcyear();
        $periods = Periods::where('acyear', $acyearId)->get();
        
        $timetables = $fyugptimetables = [];
        foreach($periods as $period) {
            $interchanges = TimetableInterchange::where('user_id', $userId)->where('date', date('Y-m-d'))->where('period_id', $period->id)->get();
            if($interchanges->count() > 0) {
                foreach($interchanges as $interchange) {
                    $timetables[] = [
                            'course_id' => $interchange->course_id,
                            'course_name' => $interchange->course->course_name,
                            'semester_id' => $interchange->semester_id,
                            'semester_name' => $interchange->semester->semester_name,
                            'subject_id' => $interchange->subject_id, 
                            'subject_name' => $interchange->subject->subject_name, 
                            'period_id' => $period->id,
                            'period_name' => $period->period,
                            'type' => 1
                        ];
                }
            } else {
                $timeperiods = Timetable::where('acyear', $acyearId)->where('user_id', $userId)->where('day_of_week', $day)->where('period_id', $period->id)->get();
                if($timeperiods->count() > 0) {
                    foreach($timeperiods as $timeperiod) {
                        $check = TimetableInterchange::where('course_id', $timeperiod->course)->where('semester_id', $timeperiod->semester)->where('date', date('Y-m-d'))->where('period_id', $period->id)->get();
                        if($check->count()==0) {
                            $timetables[] = [
                                    'course_id' => $timeperiod->course,
                                    'course_name' => $timeperiod->cnames->course_name,
                                    'semester_id' => $timeperiod->semester,
                                    'semester_name' => $timeperiod->semnames->semester_name,
                                    'subject_id' => $timeperiod->subject_id, 
                                    'subject_name' => $timeperiod->subject->subject_name, 
                                    'period_id' => $period->id,
                                    'period_name' => $period->period,
                                    'type' => 0
                                ];
                        }
                    }
                }
            }
            
            $fyugpinterchanges = FyugpTimetableInterchange::join('staff_subject', function($join) {
                    $join->on('fyugp_timetable_inter.course_id', '=', 'staff_subject.course_id')
                        ->on('fyugp_timetable_inter.semester_id', '=', 'staff_subject.semester_id')
                        ->whereNull('staff_subject.deleted_at');
                })
                ->join('subjects', function($join) {
                    $join->on('staff_subject.subject_id', '=', 'subjects.subject_id')
                        ->on('fyugp_timetable_inter.category_id', '=', 'subjects.subject_categ');
                })
                ->where('fyugp_timetable_inter.acyear_id', $acyearId)
                ->where('staff_subject.acyear_id', $acyearId)
                ->where('staff_subject.user_id', $userId)
                ->where('fyugp_timetable_inter.date', date('Y-m-d'))
                ->where('fyugp_timetable_inter.period_id', $period->id)
                ->select('fyugp_timetable_inter.*','staff_subject.subject_id','subjects.subject_name')
                ->get();
                
            if($fyugpinterchanges->count() > 0) {
                foreach($fyugpinterchanges as $fyugpinterchange) {
                    $fyugptimetables[] = [
                            'data_id' => $fyugpinterchange->id,
                            'course_id' => $fyugpinterchange->course_id,
                            'course_name' => $fyugpinterchange->course->course_name,
                            'semester_id' => $fyugpinterchange->semester_id,
                            'semester_name' => $fyugpinterchange->semester->semester_name,
                            'category_id' => $fyugpinterchange->category_id,
                            'category_name' => $fyugpinterchange->category->category,
                            'subject_id' => $fyugpinterchange->subject_id,
                            'subject_name' => $fyugpinterchange->subject_name,
                            'period_id' => $period->id,
                            'period_name' => $period->period,
                            'type' => 1
                        ];
                }
            } else {
                $fyugptimeperiods = FyugpTimetable::where('acyear', $acyearId)
                    ->where('user_id', $userId)
                    ->where('day_of_week', $day)
                    ->where('period_id', $period->id)
                    ->orderBy('period_id', 'ASC')
                    ->get();
                    
                if($fyugptimeperiods->count() > 0) {
                    foreach($fyugptimeperiods as $fyugptimeperiod) {
                        $subject_query = StaffSubject::where('user_id', $userId)
                            ->where('category_id', $fyugptimeperiod->categ_id);
                        if($fyugptimeperiod->subject_id != null && $fyugptimeperiod->subject_id > 0) {
                            $subject_query->where('subject_id', $fyugptimeperiod->subject_id);
                        }    
                            
                        if($fyugptimeperiod->group_id==null || $fyugptimeperiod->group_id==0) {
                            $subject_query->where('course_id', $fyugptimeperiod->course)
                                ->where('semester_id', $fyugptimeperiod->semester);
                        } else {
                            $courses = DB::table('group_courses')->where('group_id', $fyugptimeperiod->group_id)->pluck('course_id')->toArray();
                            $semesters = DB::table('semesters')->whereIn('course_id', $courses)->where('semester_name', $fyugptimeperiod->semesterdet->semester_name)->pluck('semester_id')->toArray();
                            $subject_query->whereIn('semester_id', $semesters);
                        }
                        
                        $subject = $subject_query->first();
                        
                        $subject_id=$subject_name='';
                        if($subject) {
                            $subject_id = $subject->subject_id;
                            $subject_name = $subject->subject->subject_name;
                        }
                        
                        $fyugptimetables[] = [
                            'data_id' => $fyugptimeperiod->id,
                            'course_id' => $fyugptimeperiod->course,
                            'course_name' => $fyugptimeperiod->coursedet->course_name,
                            'semester_id' => $fyugptimeperiod->semester,
                            'semester_name' => $fyugptimeperiod->semesterdet->semester_name,
                            'category_id' => $fyugptimeperiod->categ_id, 
                            'category_name' => $fyugptimeperiod->category->category,
                            'subject_id' => $subject_id, 
                            'subject_name' => $subject_name,
                            'group_id' => $fyugptimeperiod->group_id,
                            'group_name' => $fyugptimeperiod->group->name,
                            'period_id' => $period->id,
                            'period_name' => $period->period,
                            'type' => 0
                        ];
                    }
                }
            }
        }
        
        return view('staff.dashboard.timetable', compact('timetables', 'fyugptimetables'));
    }
    
    public function getSemesters($courseId) {
        
        $semesterList = [];
        $semesters = Semester::where('course_id', $courseId)->where('status', 1)->get();
        
        if($semesters) {
            foreach($semesters as $semester) {
                $semesterList[$semester->semester_id] = $semester->semester_name;
            }
        }
        
        return json_encode($semesterList);
    }
    
    public function getCourseSubjects(Request $request) {
        $course = $request->get('course');
        $semester = $request->get('semester');
        $staff = $request->get('staff');
        
        $acyearId = getAcyear();
        $subjectList = [];
        if($course!='' && $semester!='' && $staff!='') {
            $staffsubjects = StaffSubject::where('acyear_id', $acyearId)->where('course_id', $course)->where('semester_id', $semester)->where('user_id', $staff)->get();
            if($staffsubjects->count() > 0) {
                foreach($staffsubjects as $staffsubject) {
                    $subjectList[$staffsubject->subject_id] = $staffsubject->subject->subject_code.' - '.$staffsubject->subject->subject_name;
                }
            }
        }
        
        return json_encode($subjectList);
    }
}
