<?php

namespace App\Http\Controllers\Staff;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;

use App\Models\User;
use App\Models\Periods;
use App\Models\Timetable;
use App\Models\TimetableInterchange;
use App\Models\StudentAttendance;
use App\Models\Studentleavetype;
use App\Models\StudentExemption;
use App\Models\PriorAttendance;
use App\Models\Promote;
use App\Models\Course;
use App\Models\Semester;
use App\Models\Subject;
use App\Models\Student;
use App\Models\AcHoliday;
use App\Models\ClassTeacher;
use App\Models\StaffSubject;
use App\Models\FyugpTimetable;
use App\Models\SubCategory;
use DB;
use Carbon\Carbon;

class StudentAttendances extends Controller
{
    public function hourly(Request $request){
        
        try{
            $user = Auth::user();
            $cdate = $request->input('cdate') ?? date('Y-m-d');
            $acyearId = getAcyear();
            $periods = Periods::where('acyear', $acyearId)->get();
            $startOfWeek = Carbon::parse($cdate)->startOfWeek(Carbon::MONDAY);
            $endOfWeek = Carbon::parse($cdate)->endOfWeek(Carbon::SATURDAY);
            $currentDate = $startOfWeek;
            
            $timetable = Timetable::where('acyear', $acyearId)->where('user_id', $user->user_id)->get();
            $attendances = StudentAttendance::whereBetween('date',[$startOfWeek->toDateString(), $endOfWeek->toDateString()])->select('course_id','semester_id','subject_id','period_id','date',DB::raw("COUNT(student_id) AS total"))->groupBy('course_id','semester_id','subject_id','period_id','date')->get();
            $data = $dates = [];
        
            while ($currentDate->lte($endOfWeek)) {
                $dateString = $currentDate->toDateString();
                $dayName = $currentDate->format('l');
                $dates[] = $dateString;
                
                $holiday = AcHoliday::where('start_date', '<=', $dateString)->where('end_date', '>=', $dateString)->where('student', 1)->count();
                
                foreach($periods as $period) {
                    $interchanges = TimetableInterchange::where('user_id', $user->user_id)->where('period_id', $period->id)->where('date', $dateString)->get();
                    if($interchanges->count() > 0) {
                        foreach($interchanges as $interchange) {
                            if($interchange->subject->subject_categ==10) {
                                $students = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')->where('promotes.course_id', $interchange->course_id)->where('promotes.semester_id', $interchange->semester_id)->where('promotes.ac_id', $acyearId)->where('students.seclang', $interchange->subject->seclang)->distinct('promotes.student_id')->count('promotes.student_id');
                            } else {
                                $students = Promote::where('course_id', $interchange->course_id)->where('semester_id', $interchange->semester_id)->where('ac_id', $acyearId)->distinct('student_id')->count('student_id');
                            }
                            
                            $atcount = $attendances->where('date', $dateString)->where('course_id', $interchange->course_id)->where('semester_id', $interchange->semester_id)->where('subject_id', $interchange->subject_id)->where('period_id', $period->id)->pluck('total')->first();
                            
                            $marked=0;
                            if($atcount>0){
                                if($atcount>=$students){
                                    $marked=1;
                                }else{
                                    $marked=2;
                                }
                            }
                            
                            $data[$dateString][$period->id][] = [
                                    'subject_id' => $interchange->subject_id, 
                                    'subject_name' => $interchange->subject->subject_name, 
                                    'course_id' => $interchange->course_id,
                                    'course_name' => $interchange->course->course_name,
                                    'semester_id' => $interchange->semester_id,
                                    'semester_name' => $interchange->semester->semester_name,
                                    'marked' => $marked,
                                    'holiday' => $holiday,
                                    'type' => 1
                                ];
                        }
                    } else {
                        $timeperiods = $timetable->where('period_id', $period->id)->where('day_of_week',$dayName);
                        foreach($timeperiods as $timeperiod) {
                            $check = TimetableInterchange::where('course_id', $timeperiod->course)->where('semester_id', $timeperiod->semester)->where('date', $dateString)->where('period_id', $period->id)->get();
                            if($check->count()==0) {
                                $subcateg = Subject::where('subject_id',$timeperiod->subject_id)->first();
                                if($subcateg->subject_categ == '10') {
                                    $students = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')->where('promotes.course_id', $timeperiod->course)->where('promotes.semester_id', $timeperiod->semester)->where('promotes.ac_id', $acyearId)->where('students.seclang',$subcateg->seclang)->distinct('promotes.student_id')->count('promotes.student_id');
                                } else {
                                    $students = Promote::where('course_id', $timeperiod->course)->where('semester_id', $timeperiod->semester)->where('ac_id', $acyearId)->distinct('student_id')->count('student_id');
                                }
                                
                                $atcount = $attendances->where('date',$dateString)->where('course_id',$timeperiod->course)->where('semester_id',$timeperiod->semester)->where('subject_id',$timeperiod->subject_id)->where('period_id',$period->id)->pluck('total')->first();
                                $marked=0;
                                if($atcount>0){
                                    if($atcount>=$students){
                                        $marked=1;
                                    }else{
                                        $marked=2;
                                    }
                                }
                                
                                $data[$dateString][$period->id][] = [
                                        'course_id' => $timeperiod->course,
                                        'course_name' => $timeperiod->cnames->course_name,
                                        'semester_id' => $timeperiod->semester,
                                        'semester_name' => $timeperiod->semnames->semester_name,
                                        'subject_id' => $timeperiod->subject_id, 
                                        'subject_name' => $timeperiod->subject->subject_name, 
                                        'marked' => $marked,
                                        'holiday' => $holiday,
                                        'type' => 0
                                    ];
                            }
                        }
                    }
                }
                
                $currentDate = $currentDate->addDay();
            }
            
            return view('staff.student.hourly', compact('cdate','periods','dates','data'));
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function add(Request $request){
        try{
            $date = $request->date ?? null;
            $course_id = $request->course ?? null;
            $semester_id = $request->semester ?? null;
            $subject_id = $request->subject ?? null;
            $period_id = $request->period ?? null;
            
            $acyearId = getAcyear();
            
            if($date!=null && $course_id!=null && $semester_id!=null && $subject_id!=null && $period_id!=null){
                $holiday = AcHoliday::where('start_date', '<=', $date)->where('end_date', '>=', $date)->where('student', 1)->count();
                if($holiday == 0){
                    $course = Course::find($course_id);
                    $semester = Semester::find($semester_id);
                    $subject = Subject::find($subject_id);
                    $period = Periods::find($period_id);
                    $subcateg = Subject::where('subject_id',$subject_id)->first();
                    if($subcateg->subject_categ == '10') {
                        $promotes = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')
                            ->where('promotes.course_id', $course_id)
                            ->where('promotes.semester_id', $semester_id)
                            ->where('promotes.ac_id', $acyearId)
                            ->where('students.seclang',$subcateg->seclang)
                            ->select('students.student_id','students.adno','students.rollno','students.clno','students.name')
                            ->groupBy('promotes.student_id')
                            ->orderBy('students.clno', 'ASC')
                            ->get();
                        $attendance = StudentAttendance::where('date', $date)->where('course_id', $course_id)->where('semester_id', $semester_id)->where('subject_id', $subject_id)->where('period_id', $period_id)->get();
                    } else {
                        $promotes = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')
                            ->where('promotes.course_id', $course_id)
                            ->where('promotes.semester_id', $semester_id)
                            ->where('promotes.ac_id', $acyearId)
                            ->select('students.student_id','students.adno','students.rollno','students.clno','students.name')
                            ->groupBy('promotes.student_id')
                            ->orderBy('students.clno', 'ASC')
                            ->get();
                        $attendance = StudentAttendance::where('date', $date)->where('course_id', $course_id)->where('semester_id', $semester_id)->where('subject_id', $subject_id)->where('period_id', $period_id)->get();
                    }
                    
                    $students = [];
                    foreach($promotes as $promote){
                        $attend='';
                        $atdata = $attendance->where('student_id', $promote->student_id)->first();
                        if($atdata){
                            $attend = $atdata->status;
                        }
                        
                        $students[] = [
                                        'student_id' => $promote->student_id,
                                        'adno' => $promote->adno,
                                        'rollno' => $promote->rollno,
                                        'name' => $promote->name,
                                        'status' => $attend,
                                    ];
                    }
                    
                    return view('staff.student.add',compact('date', 'course', 'semester', 'subject', 'period', 'students', 'attendance'));
                }else{
                    return back()->with(['message'=>'Selected Date is Holiday !','alert-type'=>'error']);
                }
            }else{
                return back()->with(['message'=>'Invalid data !','alert-type'=>'error']);
            }
            
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function save(Request $request){
        try{
            $validated = $request->validate([
                    'course' => 'required|numeric',
                    'semester' => 'required|numeric',
                    'subject' => 'required|numeric',
                    'date' => 'required|date',
                    'period' => 'required|numeric',
                    'posted' => 'array',
                    'posted.*' => 'nullable|numeric'
                ]);
            
            $acyearId = getAcyear();
            $userId = Auth::id();
            $postedStudents = $validated['posted'] ?? [];
            
            foreach($request->studentids as $student){
                $status = 'A';
                if(in_array($student, $postedStudents)) { $status = 'P'; }
                StudentAttendance::firstOrCreate([
                        'ac_id' => $acyearId,
                        'course_id' => $validated['course'],
                        'semester_id' => $validated['semester'],
                        'date' => $validated['date'],
                        'period_id' => $validated['period'],
                        'student_id' => $student
                    ],[
                        'subject_id' => $validated['subject'],
                        'status' => $status,
                        'created_by' => $userId
                    ]);
            }
            
            return redirect()->route('staff.student.attendance',['cdate'=>$validated['date']])->with(['message'=>'Attendance Added Successfully !','alert-type'=>'success']);
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function details(Request $request){
        
        $course_id = $request->course ?? null;
        $semester_id = $request->semester ?? null;
        $subject_id = $request->subject ?? null;
        $date = $request->date ?? null;
        $period_id = $request->period ?? null;
        $student_id = $request->sid ?? null;
        
        if($course_id!=null && $semester_id!=null && $subject_id!=null && $date!=null && $period_id!=null && $student_id!=null){
            $course = Course::find($course_id);
            $semester = Semester::find($semester_id);
            $subject = Subject::find($subject_id);
            $period = Periods::find($period_id);
            $student = Student::find($student_id);
            
            $attendance = StudentAttendance::where('date', $date)->where('course_id', $course_id)->where('semester_id', $semester_id)->where('subject_id', $subject_id)->where('period_id', $period_id)->where('student_id', $student_id)->first();
            $status=$reason='';
            if($attendance){
                $status = $attendance->status;
                $reason = $attendance->reason;
            }
            
            return view('staff.student.details',compact('course', 'semester', 'subject', 'period', 'date', 'student', 'attendance', 'status', 'reason'));
        }
        
        return false;
    }
    
    public function leave(Request $request){
        try{
            $course = $request->course ?? null;
            $semester = $request->semester ?? null;
            $subject = $request->subject ?? null;
            $date = $request->date ?? null;
            $period = $request->period ?? null;
            $student = $request->student ?? null;
            $status = $request->status ?? null;
            $reason = $request->reason ?? null;
            
            $acyearId = getAcyear();
            
            if($course!=null && $semester!=null && $subject!=null && $date!=null && $period!=null && $student!=null && $status!=null && $acyearId!=null){
                StudentAttendance::updateOrCreate([
                        'ac_id' => $acyearId,
                        'course_id' => $course,
                        'semester_id' => $semester,
                        'subject_id' => $subject,
                        'date' => $date,
                        'period_id' => $period,
                        'student_id' => $student
                    ],[
                        'status' => $status,
                        'reason' => $reason
                    ]);
                        
                $msgArray = ['message'=>'Attendance Added Successfully !','alert-type'=>'success'];
            }else{
                $msgArray = ['message'=>'Attendance could not be Added !','alert-type'=>'error'];
            }
            
            return redirect()->route('staff.student.attendance.add',['date'=>$date, 'course'=>$course, 'semester'=>$semester, 'subject'=>$subject, 'period'=>$period])->with($msgArray);
            
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function all(Request $request) {
        try {
            $acyearId = getAcyear();
            $userId = Auth::id();
            $courses = ClassTeacher::join('courses', 'classteacher.course_id', '=', 'courses.course_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->select('courses.*')->groupBy('classteacher.course_id')->get();
            if($courses->count() > 0) {
                $courseId = $request->course ?? null;
                $semesters = [];
                if($courseId!=null) {
                    $semesters = ClassTeacher::join('semesters', 'classteacher.semester_id', '=', 'semesters.semester_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->where('classteacher.course_id', $courseId)->select('semesters.*')->groupBy('classteacher.semester_id')->get();
                }
                
                $semesterId = $request->semester ?? null;
                $date = $request->date ?? date('Y-m-d');
                $periods = Periods::where('acyear', $acyearId)->get();
                $startOfWeek = Carbon::parse($date)->startOfWeek(Carbon::MONDAY);
                $endOfWeek = Carbon::parse($date)->endOfWeek(Carbon::SATURDAY);
                $currentDate = $startOfWeek;
                $data = $dates = [];
                if($courseId!=null && $semesterId!=null && $date!=null) {
                    $timetable = Timetable::where('acyear', $acyearId)->where('course', $courseId)->where('semester', $semesterId)->get();
                    $attendances = StudentAttendance::whereBetween('date',[$startOfWeek->toDateString(), $endOfWeek->toDateString()])->where('course_id', $courseId)->where('semester_id', $semesterId)->select('subject_id','period_id','date','student_id')->get();
                    while ($currentDate->lte($endOfWeek)) {
                        $dateString = $currentDate->toDateString();
                        $dayName = $currentDate->format('l');
                        $dates[] = $dateString;
                        
                        $holiday = AcHoliday::where('start_date', '<=', $dateString)->where('end_date', '>=', $dateString)->where('student', 1)->count();
                        
                        foreach($periods as $period) {
                            $interchanges = TimetableInterchange::where('course_id', $courseId)->where('semester_id', $semesterId)->where('period_id', $period->id)->where('date', $dateString)->get();
                            if($interchanges->count() > 0) {
                                foreach($interchanges as $interchange) {
                                    if($interchange->subject->subject_categ==10) {
                                        $students = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')->where('promotes.course_id', $interchange->course_id)->where('promotes.semester_id', $interchange->semester_id)->where('promotes.ac_id', $acyearId)->where('students.seclang', $interchange->subject->seclang)->distinct('promotes.student_id')->count('promotes.student_id');
                                    } else {
                                        $students = Promote::where('course_id', $interchange->course_id)->where('semester_id', $interchange->semester_id)->where('ac_id', $acyearId)->distinct('student_id')->count('student_id');
                                    }
                                    
                                    $atcount = $attendances->where('date', $dateString)->where('course_id', $interchange->course_id)->where('semester_id', $interchange->semester_id)->where('subject_id', $interchange->subject_id)->where('period_id', $period->id)->pluck('total')->first();
                            
                                    $marked=0;
                                    if($atcount>0){
                                        if($atcount>=$students){
                                            $marked=1;
                                        }else{
                                            $marked=2;
                                        }
                                    }
                                    
                                    $data[$dateString][$period->id][] = [
                                            'course_id' => $interchange->course_id,
                                            'course_name' => $interchange->course->course_name,
                                            'subject_id' => $interchange->subject_id, 
                                            'subject_name' => $interchange->subject->subject_name, 
                                            'semester_id' => $interchange->semester_id,
                                            'semester_name' => $interchange->semester->semester_name,
                                            'staff_id' => $interchange->user_id,
                                            'staff_name' => $interchange->user->name,
                                            'marked' => $marked,
                                            'holiday' => $holiday,
                                            'type' => 1
                                        ];
                                }
                            } else {
                                $timeperiods = $timetable->where('period_id',$period->id)->where('day_of_week',$dayName);
                                if($timeperiods->count() > 0) {
                                    foreach($timeperiods as $timeperiod) {
                                        $check = TimetableInterchange::where('course_id', $timeperiod->course)->where('semester_id', $timeperiod->semester)->where('date', $dateString)->where('period_id', $period->id)->get();
                                        if($check->count()==0) {
                                            $marked=0;
                                            $subcateg = Subject::where('subject_id',$timeperiod->subject_id)->first();
                                            if($subcateg->subject_categ == '10') {
                                                $students = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')->where('promotes.course_id', $timeperiod->course)->where('promotes.semester_id', $timeperiod->semester)->where('promotes.ac_id', $acyearId)->where('students.seclang',$subcateg->seclang)->distinct('promotes.student_id')->count('promotes.student_id');
                                            } else {
                                                $students = Promote::where('course_id', $timeperiod->course)->where('semester_id', $timeperiod->semester)->where('ac_id', $acyearId)->distinct('student_id')->count('student_id');
                                            }
                                            
                                            $atcount = $attendances->where('date',$dateString)->where('subject_id',$timeperiod->subject_id)->where('period_id',$period->id)->count();
                                            if($atcount>0){
                                                if($atcount>=$students){
                                                    $marked=1;
                                                }else{
                                                    $marked=2;
                                                }
                                            }
                                            
                                            $data[$dateString][$period->id][] = [
                                                    'course_id' => $timeperiod->course,
                                                    'course_name' => $timeperiod->cnames->course_name,
                                                    'semester_id' => $timeperiod->semester,
                                                    'semester_name' => $timeperiod->semnames->semester_name,
                                                    'subject_id' => $timeperiod->subject_id,
                                                    'subject_name' => $timeperiod->subject->subject_name,
                                                    'staff_id' => $timeperiod->user_id,
                                                    'staff_name' => $timeperiod->unames->name,
                                                    'marked' => $marked,
                                                    'holiday' => $holiday,
                                                    'type' => 0
                                                ];
                                        }
                                    }
                                }
                            }
                        }
                        
                        $currentDate = $currentDate->addDay();
                    }
                }
                
                return view('staff.student.all', compact('courses', 'courseId', 'semesters', 'semesterId', 'date', 'dates', 'periods', 'data'));
            }else{
                return back()->with(['message'=>'Permission denied !','alert-type'=>'error']);
            }
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function add_all(Request $request){
        try{
            $date = $request->date ?? null;
            $course_id = $request->course ?? null;
            $semester_id = $request->semester ?? null;
            $subject_id = $request->subject ?? null;
            $period_id = $request->period ?? null;
            $staff_id = $request->staff ?? null;
            $acyearId = getAcyear();
            
            if($date!=null && $course_id!=null && $semester_id!=null && $subject_id!=null && $period_id!=null){
                $holiday = AcHoliday::where('start_date', '<=', $date)->where('end_date', '>=', $date)->where('student', 1)->count();
                if($holiday == 0){
                    $course = Course::find($course_id);
                    $semester = Semester::find($semester_id);
                    $subject = Subject::find($subject_id);
                    $period = Periods::find($period_id);
                    $staff = User::find($staff_id);
                    $subcateg = Subject::where('subject_id',$subject_id)->first();
                    if($subcateg->subject_categ == '10') {
                        $promotes = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')
                            ->where('promotes.course_id', $course_id)
                            ->where('promotes.semester_id', $semester_id)
                            ->where('promotes.ac_id', $acyearId)
                            ->where('students.seclang',$subcateg->seclang)
                            ->select('students.student_id','students.adno','students.rollno','students.clno','students.name')
                            ->groupBy('promotes.student_id')
                            ->orderBy('students.clno', 'ASC')
                            ->get();
                    } else {
                        $promotes = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')
                            ->where('promotes.course_id', $course_id)
                            ->where('promotes.semester_id', $semester_id)
                            ->where('promotes.ac_id', $acyearId)
                            ->select('students.student_id','students.adno','students.rollno','students.clno','students.name')
                            ->groupBy('promotes.student_id')
                            ->orderBy('students.clno', 'ASC')
                            ->get();
                    }
                    $attendance = StudentAttendance::where('date', $date)->where('course_id', $course_id)->where('semester_id', $semester_id)->where('subject_id', $subject_id)->where('period_id', $period_id)->get();
                    
                    $students = [];
                    foreach($promotes as $promote){
                        $attend='';
                        $atdata = $attendance->where('student_id', $promote->student_id)->first();
                        if($atdata){
                            $attend = $atdata->status;
                        }
                        
                        $students[] = [
                                'student_id' => $promote->student_id,
                                'adno' => $promote->adno,
                                'rollno' => $promote->rollno,
                                'name' => $promote->name,
                                'status' => $attend,
                            ];
                    }
                    
                    return view('staff.student.add_all',compact('date', 'course', 'semester', 'subject', 'period', 'staff', 'students', 'attendance'));
                }else{
                    return back()->with(['message'=>'Selected Date is Holiday !','alert-type'=>'error']);
                }
            }else{
                return back()->with(['message'=>'Invalid data !','alert-type'=>'error']);
            }
            
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function all_save(Request $request){
        try{
            $validated = $request->validate([
                    'course' => 'required|numeric',
                    'semester' => 'required|numeric',
                    'subject' => 'required|numeric',
                    'date' => 'required|date',
                    'period' => 'required|numeric',
                    'posted' => 'array|present',
                    'posted.*' => 'nullable|numeric'
                ]);
            
            $acyearId = getAcyear();
            $userId = Auth::id();
            
            foreach($validated['posted'] as $student){
                StudentAttendance::updateOrCreate([
                        'ac_id' => $acyearId,
                        'course_id' => $validated['course'],
                        'semester_id' => $validated['semester'],
                        'date' => $validated['date'],
                        'period_id' => $validated['period'],
                        'student_id' => $student
                    ],[
                        'subject_id' => $validated['subject'],
                        'status' => 'P',
                        'created_by' => $userId
                    ]);
            }
            
            return redirect()->route('staff.student.attendance.all',['course'=>$validated['course'], 'semester'=>$validated['semester'], 'date'=>$validated['date']])->with(['message'=>'Attendance Added Successfully !','alert-type'=>'success']);
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function all_details(Request $request){
        
        $course_id = $request->course ?? null;
        $semester_id = $request->semester ?? null;
        $subject_id = $request->subject ?? null;
        $staff_id = $request->staff ?? null;
        $date = $request->date ?? null;
        $period_id = $request->period ?? null;
        $student_id = $request->sid ?? null;
        
        if($course_id!=null && $semester_id!=null && $subject_id!=null && $staff_id!=null && $date!=null && $period_id!=null && $student_id!=null){
            $course = Course::find($course_id);
            $semester = Semester::find($semester_id);
            $subject = Subject::find($subject_id);
            $staff = User::find($staff_id);
            $period = Periods::find($period_id);
            $student = Student::find($student_id);
            
            $leaves = Studentleavetype::all();
            
            $attendance = StudentAttendance::where('date', $date)->where('course_id', $course_id)->where('semester_id', $semester_id)->where('subject_id', $subject_id)->where('period_id', $period_id)->where('student_id', $student_id)->first();
            $status=$reason=$ltype='';
            if($attendance){
                $status = $attendance->status;
                $reason = $attendance->reason;
                $ltype = $attendance->leave_id;
            }
            
            return view('staff.student.all_details',compact('course', 'semester', 'subject', 'staff', 'period', 'date', 'student', 'attendance', 'leaves', 'ltype', 'status', 'reason'));
        }
        
        return false;
    }
    
    public function all_leave(Request $request){
        try{
            $course = $request->course ?? null;
            $semester = $request->semester ?? null;
            $subject = $request->subject ?? null;
            $staff = $request->staff ?? null;
            $date = $request->date ?? null;
            $period = $request->period ?? null;
            $student = $request->student ?? null;
            $status = $request->status ?? null;
            $leave = $request->ltype ?? null;
            $reason = $request->reason ?? null;
            
            $acyearId = getAcyear();
            
            if($course!=null && $semester!=null && $subject!=null && $staff!=null && $date!=null && $period!=null && $student!=null && $status!=null && $acyearId!=null){
                StudentAttendance::create([
                        'ac_id' => $acyearId,
                        'course_id' => $course,
                        'semester_id' => $semester,
                        'subject_id' => $subject,
                        'date' => $date,
                        'period_id' => $period,
                        'student_id' => $student,
                        'status' => $status,
                        'leave_id' => $leave,
                        'reason' => $reason,
                    ]);
                    
                $msgArray = ['message'=>'Attendance Added Successfully !','alert-type'=>'success'];
            }else{
                $msgArray = ['message'=>'Attendance could not be Added !','alert-type'=>'error'];
            }
            
            return redirect()->route('staff.student.attendance.all.add',['date'=>$date, 'course'=>$course, 'semester'=>$semester, 'subject'=>$subject, 'period'=>$period, 'staff'=>$staff])->with($msgArray);
            
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function exemption(Request $request){
        try{
            $acyearId = getAcyear();
            $userId = Auth::id();
            $courses = ClassTeacher::join('courses', 'classteacher.course_id', '=', 'courses.course_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->select('courses.*')->groupBy('classteacher.course_id')->get();
            if($courses->count() > 0){
                $courseId = $request->course ?? null;
                $semesters = [];
                if($courseId!=null){
                    $semesters = ClassTeacher::join('semesters', 'classteacher.semester_id', '=', 'semesters.semester_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->where('classteacher.course_id', $courseId)->select('semesters.*')->groupBy('classteacher.semester_id')->get();
                }
                $semesterId = $request->semester ?? null;
                $leaves = Studentleavetype::all();
                $leaveId = $request->leave ?? null;
                
                $query = StudentExemption::where('user_id', $userId);
                if($courseId!=null){
                    $query->where('course_id', $courseId);
                }
                if($semesterId!=null){
                    $query->where('semester_id', $semesterId);
                }
                if($leaveId!=null){
                    $query->where('leave_id', $leaveId);
                }
                
                $exemptions = $query->get();
                
                return view('staff.student.exemption', compact('courses', 'courseId', 'semesters', 'semesterId', 'leaves', 'leaveId', 'exemptions'));
            }else{
                return back()->with(['message'=>'Permission denied !','alert-type'=>'error']);
            }
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function add_exemption(Request $request){
        try{
            $acyearId = getAcyear();
            $userId = Auth::id();
            $courses = ClassTeacher::join('courses', 'classteacher.course_id', '=', 'courses.course_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->select('courses.*')->groupBy('classteacher.course_id')->get();
            if($courses->count() > 0){
                $courseId = $request->course ?? null;
                $semesters = [];
                if($courseId!=null){
                    $semesters = ClassTeacher::join('semesters', 'classteacher.semester_id', '=', 'semesters.semester_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->where('classteacher.course_id', $courseId)->select('semesters.*')->groupBy('classteacher.semester_id')->get();
                }
                $semesterId = $request->semester ?? null;
                $students = [];
                if($courseId!=null && $semesterId!=null){
                    $students = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')
                                    ->where('promotes.course_id', $courseId)
                                    ->where('promotes.semester_id', $semesterId)
                                    ->where('promotes.ac_id', $acyearId)
                                    ->select('students.student_id','students.adno','students.name')
                                    ->groupBy('promotes.student_id')
                                    ->orderBy('students.name', 'ASC')
                                    ->get();
                }
                $leaves = Studentleavetype::all();
                $fdate = $request->fdate ?? date('Y-m-d');
                $tdate = $request->tdate ?? date('Y-m-d');
                
                return view('staff.student.add_exemption', compact('courses', 'semesters', 'students', 'leaves', 'fdate', 'tdate'));
            }else{
                return back()->with(['message'=>'Permission denied !','alert-type'=>'error']);
            }
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function save_exemption(Request $request){
        try {
            $validated = $request->validate([
                'course' => 'required|exists:courses,course_id',
                'semester' => 'required|exists:semesters,semester_id',
                'student' => 'required|exists:students,student_id',
                'leave' => 'required|exists:student_leave_setting,stuleav_id',
                'from_date' => [
                    'required',
                    'date',
                    'before_or_equal:to_date',
                    'after_or_equal:' . now()->subDays(14)->toDateString(),
                ],
                'to_date' => [
                    'required',
                    'date',
                    'after:from_date',
                    'before_or_equal:today',
                ],
                'reason' => 'required',
            ], [
                'from_date.after_or_equal' => 'The from date must not be earlier than 14 days before today.',
            ]);
        
            $acyearId = getAcyear();
            $userId = Auth::id();
        
            $saved = StudentExemption::firstOrCreate([
                'ac_id' => $acyearId,
                'course_id' => $validated['course'],
                'semester_id' => $validated['semester'],
                'student_id' => $validated['student'],
                'leave_id' => $validated['leave'],
                'fdate' => $validated['from_date'],
                'tdate' => $validated['to_date']
            ],[
                'reason' => $validated['reason'],
                'user_id' => $userId,
            ]);
            
            if($saved){
                $startOfWeek = Carbon::parse($validated['from_date']);
                $endOfWeek = Carbon::parse($validated['to_date']);
                $currentDate = $startOfWeek;
                $timetable = Timetable::where('acyear', $acyearId)->where('course', $validated['course'])->where('semester', $validated['semester'])->get();
                
                while ($currentDate->lte($endOfWeek)) {
                    $dateString = $currentDate->toDateString();
                    $dayName = $currentDate->format('l');
                    $holiday = AcHoliday::where('start_date', '<=', $dateString)->where('end_date', '>=', $dateString)->where('student', 1)->count();
                    if($holiday==0){
                        $timeperiods = $timetable->where('day_of_week', $dayName);
                        foreach($timeperiods as $timeperiod){
                            StudentAttendance::updateOrCreate([
                                    'ac_id' => $acyearId,
                                    'course_id' => $validated['course'],
                                    'semester_id' => $validated['semester'],
                                    'subject_id' => $timeperiod->subject_id,
                                    'date' => $dateString,
                                    'period_id' => $timeperiod->period_id,
                                    'student_id' => $validated['student'],
                                ],[
                                    'status' => 'P',
                                    'leave_id' => $validated['leave'],
                                    'reason' => $validated['reason']
                                ]);
                        }
                    }
                    $currentDate = $currentDate->addDay();
                }
                $msgArray = ['message' => 'Exemption Added Successfully.', 'alert-type' => 'success'];
            }else{
                $msgArray = ['message' => 'Exemption could not be Added.', 'alert-type' => 'error'];
            }
                
            return redirect()->route('staff.student.attendance.exemption.add')->with($msgArray);
            
        } catch (ValidationException $e) {
            // Log validation errors for debugging
            Log::error('Validation failed: ' . json_encode($e->errors()));
        
            // Redirect back with validation errors
            return back()
                ->withErrors($e->errors())
                ->withInput();
        
        } catch (\Exception $e) {
            // Log general errors
            Log::error('Failed to fetch Data: ' . $e->getMessage());
        
            return back()->with(['message' => 'Failed to load Data.', 'alert-type' => 'error']);
        }
    }
    
    public function prior(Request $request){
        try{
            $acyearId = getAcyear();
            $userId = Auth::id();
            $courses = ClassTeacher::join('courses', 'classteacher.course_id', '=', 'courses.course_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->select('courses.*')->groupBy('classteacher.course_id')->get();
            if($courses->count() > 0){
                $courseId = $request->course ?? null;
                $semesters = [];
                if($courseId!=null){
                    $semesters = ClassTeacher::join('semesters', 'classteacher.semester_id', '=', 'semesters.semester_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->where('classteacher.course_id', $courseId)->select('semesters.*')->groupBy('classteacher.semester_id')->get();
                }
                $semesterId = $request->semester ?? null;
                $leaves = Studentleavetype::all();
                $leaveId = $request->leave ?? null;
                
                $query = PriorAttendance::where('user_id', $userId);
                if($courseId!=null){
                    $query->where('course_id', $courseId);
                }
                if($semesterId!=null){
                    $query->where('semester_id', $semesterId);
                }
                if($leaveId!=null){
                    $query->where('leave_id', $leaveId);
                }
                
                $priors = $query->get();
                
                return view('staff.student.prior', compact('courses', 'courseId', 'semesters', 'semesterId', 'leaves', 'leaveId', 'priors'));
            }else{
                return back()->with(['message'=>'Permission denied !','alert-type'=>'error']);
            }
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function add_prior(Request $request){
        try{
            $acyearId = getAcyear();
            $userId = Auth::id();
            $courses = ClassTeacher::join('courses', 'classteacher.course_id', '=', 'courses.course_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->select('courses.*')->groupBy('classteacher.course_id')->get();
            if($courses->count() > 0){
                $courseId = $request->course ?? null;
                $semesters = [];
                if($courseId!=null){
                    $semesters = ClassTeacher::join('semesters', 'classteacher.semester_id', '=', 'semesters.semester_id')->where('classteacher.ac_id', $acyearId)->where('classteacher.user_id', $userId)->where('classteacher.course_id', $courseId)->select('semesters.*')->groupBy('classteacher.semester_id')->get();
                }
                $semesterId = $request->semester ?? null;
                $students = [];
                if($courseId!=null && $semesterId!=null){
                    $students = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')
                                        ->where('promotes.course_id', $courseId)
                                        ->where('promotes.semester_id', $semesterId)
                                        ->where('promotes.ac_id', $acyearId)
                                        ->select('students.student_id','students.adno','students.name')
                                        ->groupBy('promotes.student_id')
                                        ->orderBy('students.name', 'ASC')
                                        ->get();
                }
                $leaves = Studentleavetype::all();
                $fdate = $request->fdate ?? date('Y-m-d');
                $tdate = $request->tdate ?? date('Y-m-d');
                
                return view('staff.student.add_prior', compact('courses', 'semesters', 'students', 'leaves', 'fdate', 'tdate'));
            }else{
                return back()->with(['message'=>'Permission denied !','alert-type'=>'error']);
            }
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function save_prior(Request $request){
        try {
            $validated = $request->validate([
                'course' => 'required|exists:courses,course_id',
                'semester' => 'required|exists:semesters,semester_id',
                'student' => 'required|exists:students,student_id',
                'leave' => 'nullable|exists:student_leave_setting,stuleav_id',
                'from_date' => [
                    'required',
                    'date',
                    'before_or_equal:to_date',
                    
                ],
                'to_date' => [
                    'required',
                    'date',
                    'before_or_equal:'.now()->addDays(14)->toDateString(),
                ],
                'reason' => 'required',
            ], [
                'to_date.before_or_equal' => 'The to date must not be later than 14 days.',
            ]);
        
            $acyearId = getAcyear();
            $userId = Auth::id();
        
            $saved = PriorAttendance::firstOrCreate([
                'ac_id' => $acyearId,
                'course_id' => $validated['course'],
                'semester_id' => $validated['semester'],
                'student_id' => $validated['student'],
                'leave_id' => $validated['leave'],
                'fdate' => $validated['from_date'],
                'tdate' => $validated['to_date']
            ],[
                'reason' => $validated['reason'],
                'user_id' => $userId,
            ]);
            
            if($saved){
                $startOfWeek = Carbon::parse($validated['from_date']);
                $endOfWeek = Carbon::parse($validated['to_date']);
                $currentDate = $startOfWeek;
                $timetable = Timetable::where('acyear', $acyearId)->where('course', $validated['course'])->where('semester', $validated['semester'])->get();
                
                while ($currentDate->lte($endOfWeek)) {
                    $dateString = $currentDate->toDateString();
                    $dayName = $currentDate->format('l');
                    $holiday = AcHoliday::where('start_date', '<=', $dateString)->where('end_date', '>=', $dateString)->where('student', 1)->count();
                    if($holiday==0){
                        $timeperiods = $timetable->where('day_of_week', $dayName);
                        foreach($timeperiods as $timeperiod){
                            StudentAttendance::updateOrCreate([
                                    'ac_id' => $acyearId,
                                    'course_id' => $validated['course'],
                                    'semester_id' => $validated['semester'],
                                    'subject_id' => $timeperiod->subject_id,
                                    'date' => $dateString,
                                    'period_id' => $timeperiod->period_id,
                                    'student_id' => $validated['student'],
                                ],[
                                    'status' => 'A',
                                    'leave_id' => $validated['leave'],
                                    'reason' => $validated['reason']
                                ]);
                            
                        }
                    }
                    $currentDate = $currentDate->addDay();
                }
                $msgArray = ['message' => 'Attendance Added Successfully.', 'alert-type' => 'success'];
            }else{
                $msgArray = ['message' => 'Attendance could not be Added.', 'alert-type' => 'error'];
            }
                
            return redirect()->route('staff.student.attendance.prior.add')->with($msgArray);
            
        } catch (ValidationException $e) {
            // Log validation errors for debugging
            Log::error('Validation failed: ' . json_encode($e->errors()));
        
            // Redirect back with validation errors
            return back()
                ->withErrors($e->errors())
                ->withInput();
        
        } catch (\Exception $e) {
            // Log general errors
            Log::error('Failed to fetch Data: ' . $e->getMessage());
        
            return back()->with(['message' => 'Failed to load Data.', 'alert-type' => 'error']);
        }
    }
    
    
    public function fyugp_hourly(Request $request){
        try{
            $user = Auth::user();
            $cdate = $request->input('cdate') ?? date('Y-m-d');
            $acyearId = getAcyear();
            $periods = Periods::where('acyear', $acyearId)->get();
            $startOfWeek = Carbon::parse($cdate)->startOfWeek(Carbon::MONDAY);
            $endOfWeek = Carbon::parse($cdate)->endOfWeek(Carbon::FRIDAY);
            $currentDate = $startOfWeek;
            
             $timetable = FyugpTimetable::where('acyear', $acyearId)
                ->where('user_id', $user->user_id)
                ->get();
                
            $attendances = StudentAttendance::whereBetween('date',[$startOfWeek->toDateString(), $endOfWeek->toDateString()])->select('course_id','semester_id','subject_id','period_id','date',DB::raw("COUNT(student_id) AS total"))->groupBy('course_id','semester_id','subject_id','period_id','date')->get();
            
            $data = $dates = [];
            while ($currentDate->lte($endOfWeek)) {
                $dateString = $currentDate->toDateString();
                $dayName = $currentDate->format('l');
                
                $dates[] = $dateString;
                
                $holiday = AcHoliday::where('start_date', '<=', $dateString)->where('end_date', '>=', $dateString)->where('student', 1)->count();
                
                foreach ($periods as $period) {
                    $timeperiod_array = $timetable->where('period_id',$period->id)->where('day_of_week',$dayName);
                    foreach($timeperiod_array as $timeperiod) {
                        if($timeperiod) {
                            $atcount=$students=0;
                            $subject_name = $subject_id = '';
                            $staff_subject_qry = StaffSubject::where('user_id', $user->user_id)
                                ->where('course_id', $timeperiod->course)
                                ->where('semester_id', $timeperiod->semester)
                                ->where('category_id', $timeperiod->categ_id)
                                ->where('acyear_id', $acyearId);
                            if($timeperiod->subject_id != null && $timeperiod->subject_id > 0) {
                                $staff_subject_qry->where('subject_id', $timeperiod->subject_id);
                            }
                                
                            $staff_subject = $staff_subject_qry->first();
                            
                            $subject_id = $staff_subject->subject_id;
                            $subject_name = $staff_subject->subject->subject_name;
                            
                            
                            $semester_name = Semester::where('semester_id', $timeperiod->semester)->pluck('semester_name')->first();
                            if (in_array($timeperiod->categ_id, [2, 3, 4, 5, 6])) {
                                $sem_ids = Semester::where('semester_name', $semester_name)->pluck('semester_id');
                                
                            } else {
                                
                            }
                            
                        
                         /*   if (in_array($timeperiod->categ_id, ['2', '3', '4'])) {
                                $sem_ids = Semester::where('semester_name', $sem_name)->pluck('semester_id');
                                $students = Promote::join('fyugp_stud_sub', 'promotes.student_id', '=', 'fyugp_stud_sub.stud_id')
                                    ->where('promotes.ac_id', $acyearId)
                                    ->where('fyugp_stud_sub.acyear_id', $acyearId)
                                    ->where('fyugp_stud_sub.sub_id', $timeperiod->subject_id)
                                    ->where('fyugp_stud_sub.sub_cat', $timeperiod->categ_id)
                                    ->whereIn('promotes.semester_id', $sem_ids)
                                    ->whereIn('fyugp_stud_sub.sem', $sem_ids)
                                    ->distinct()
                                    ->count('promotes.student_id');
                                $atcount = $attendances->where('date',$dateString)->whereIn('semester_id',$sem_ids)->where('subject_id',$timeperiod->subject_id)->where('period_id',$period->id)->pluck('total')->first();
    
                            } else  if (in_array($timeperiod->categ_id, ['5', '6'])) {
                                $sem_ids = Semester::where('semester_name', $sem_name)->pluck('semester_id');
                                $students = Promote::join('fyugp_stud_sub', 'promotes.student_id', '=', 'fyugp_stud_sub.stud_id')
                                    ->where('promotes.ac_id', $acyearId)
                                    ->where('promotes.course_id', $timeperiod->course_id)
                                    ->where('fyugp_stud_sub.acyear_id', $acyearId)
                                    ->where('fyugp_stud_sub.sub_id', $timeperiod->subject_id)
                                    ->where('fyugp_stud_sub.sub_cat', $timeperiod->categ_id)
                                    ->whereIn('promotes.semester_id', $sem_ids)
                                    ->whereIn('fyugp_stud_sub.sem', $sem_ids)
                                    ->distinct()
                                    ->count('promotes.student_id');
                                $atcount = $attendances->where('date',$dateString)->whereIn('course_id',$timeperiod->course_id)->whereIn('semester_id',$sem_ids)->where('subject_id',$timeperiod->subject_id)->where('period_id',$period->id)->pluck('total')->first();
    
                            } else {
                                $sem_ids = Semester::where('course_id', $timeperiod->course_id)->where('semester_name', $sem_name)->pluck('semester_id');
                                $students = Promote::where('course_id', $timeperiod->course_id)
                                    ->whereIn('semester_id', $sem_ids)
                                    ->where('ac_id', $acyearId)
                                    ->distinct()
                                    ->count('student_id');
                                $atcount = $attendances->where('date',$dateString)->whereIn('course_id',$timeperiod->course_id)->whereIn('semester_id',$sem_ids)->where('subject_id',$timeperiod->subject_id)->where('period_id',$period->id)->pluck('total')->first();
                            } */
                            
                            $marked=0;
                            if($atcount>0){
                                if($atcount>=$students){
                                    $marked=1;
                                }else{
                                    $marked=2;
                                }
                            }
                            
                            $data[$dateString][$period->id][] = [
                                'data_id' => $timeperiod->id,
                                'subject_id' => $subject_id, 
                                'subject_name' => $subject_name, 
                                'category_id' => $timeperiod->categ_id,
                                'category_name' => $timeperiod->subjectcateg->category,
                                'course_id' => $timeperiod->course_id,
                                'course_name' => $timeperiod->coursedet->course_name,
                                'semester_id' => $timeperiod->semester_id,
                                'semester_name' => $semester_name,
                                'group_id' => $timeperiod->group_id,
                                'group_name' => $timeperiod->group->name,
                                'type' => $timeperiod->type,
                                'marked' => $marked,
                                'holiday' => $holiday
                            ];
                        }
                    }
                }
                
                $currentDate = $currentDate->addDay();
            }
            
            return view('staff.fyugp.hourly', compact('cdate','periods','dates','data'));
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
        
    }

    public function fyugp_add(Request $request){
        try{
            $date = $request->date ?? null;
            $period_id = $request->period ?? null;
            $data_id = $request->data_id ?? null;
            $acyearId = getAcyear();
            if($date!=null && $period_id!=null && $data_id!=null){
                $holiday = AcHoliday::where('start_date', '<=', $date)->where('end_date', '>=', $date)->where('student', 1)->count();
                if($holiday == 0) {
                    $period = Periods::find($period_id);
                    $timeperiod = FyugpTimetable::find($data_id);
                    $semester_name = $timeperiod->semesterdet->semester_name;
                    $sem_ids = Semester::where('semester_name', $semester_name)->pluck('semester_id')->toArray();
                    $staff_subject_qry = StaffSubject::where('user_id', $timeperiod->user_id)
                            ->where('course_id', $timeperiod->course)
                            ->where('semester_id', $timeperiod->semester)
                            ->where('category_id', $timeperiod->categ_id)
                            ->where('acyear_id', $timeperiod->acyear);
                    if($timeperiod->subject_id != null && $timeperiod->subject_id > 0) {
                        $staff_subject_qry->where('subject_id', $timeperiod->subject_id);
                    }
                    
                    $staffsubject = $staff_subject_qry->first();
                            
                    $subject_id = $staffsubject->subject_id;
                    $subject = $staffsubject->subject;
                    
                    $promote_qry = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')
                        ->join('courses', 'promotes.course_id', '=', 'courses.course_id');
                    if (in_array($timeperiod->categ_id, [2, 3, 4, 5, 6])) {
                        $promote_qry->join('fyugp_stud_sub', 'promotes.student_id', '=', 'fyugp_stud_sub.stud_id');
                        
                        if(in_array($timeperiod->categ_id, [5, 6])) {
                            $courses = DB::table('group_courses')->where('group_id', $timeperiod->group_id)->pluck('course_id')->toArray();
                            $promote_qry->whereIn('promotes.course_id', $courses);
                        }
                            
                        $promote_qry->where('fyugp_stud_sub.acyear_id', $timeperiod->acyear)
                            ->where('fyugp_stud_sub.sub_id', $subject_id)
                            ->where('fyugp_stud_sub.sub_cat', $timeperiod->categ_id)
                            ->whereIn('promotes.semester_id', $sem_ids)
                            ->whereIn('fyugp_stud_sub.sem', $sem_ids);
                        
                        $attendance = StudentAttendance::where('date', $date)->where('subject_id', $subject_id)->where('period_id', $period_id)->whereIn('semester_id', $sem_ids)->get();
                    } else {
                        $promote_qry = Promote::join('students', 'promotes.student_id', '=', 'students.student_id')
                            ->join('courses', 'promotes.course_id', '=', 'courses.course_id')
                            ->where('promotes.course_id', $timeperiod->course)
                            ->where('promotes.semester_id', $timeperiod->semester);
                        
                        $attendance = StudentAttendance::where('date', $date)->where('course_id', $timeperiod->course)->where('semester_id', $timeperiod->semester)->where('subject_id', $subject_id)->where('period_id', $period_id)->get();
                    }
                    
                    $promotes = $promote_qry->where('promotes.ac_id', $timeperiod->acyear)
                            ->select('students.student_id','students.adno','students.rollno','students.clno','students.name','courses.course_name','courses.course_id','promotes.semester_id')
                            ->groupBy('promotes.student_id')
                            ->orderBy('promotes.course_id', 'ASC')
                            ->orderBy('students.clno', 'ASC')
                            ->get();
                    
                    $students = [];
                    foreach($promotes as $promote) {
                        $attend='';
                        $atdata = $attendance->where('student_id', $promote->student_id)->first();
                        if($atdata){
                            $attend = $atdata->status;
                        }
                        
                        $students[] = [
                            'student_id' => $promote->student_id,
                            'adno' => $promote->adno,
                            'rollno' => $promote->rollno,
                            'name' => strtoupper($promote->name),
                            'course' => $promote->course_name,
                            'course_id'=>$promote->course_id,
                            'sem_id' =>$promote->semester_id,
                            'status' => $attend,
                        ];
                    }
                   
                    return view('staff.fyugp.add',compact('date', 'period', 'subject', 'students', 'attendance','timeperiod'));
                } else {
                    return back()->with(['message'=>'Selected Date is Holiday !','alert-type'=>'error']);
                }
            } else {
                return back()->with(['message'=>'Invalid data !','alert-type'=>'error']);
            }
            
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
    
    public function fyugp_save(Request $request){
        try{
            $validated = $request->validate([
                'subject' => 'required|numeric',
                'date' => 'required|date',
                'period' => 'required|numeric',
                'posted' => 'array',
                'posted.*' => 'nullable|numeric'
            ]);
            
            $acyearId = getAcyear();
            $userId = Auth::id();
            
            $postedStudents = $validated['posted'] ?? [];
            
            foreach($request->students as $student){
                $scourse_id = $request->input("scourse_id".$student);
                $ssem_id = $request->input("ssem_id".$student);
                $status = 'A';
                if(in_array($student, $postedStudents)) { $status = 'P'; }
                
                StudentAttendance::firstOrCreate([
                        'ac_id' => $acyearId,
                        'course_id' => $scourse_id,
                        'semester_id' => $ssem_id,
                        'date' => $validated['date'],
                        'period_id' => $validated['period'],
                        'student_id' => $student
                    ],[
                        'subject_id' => $validated['subject'],
                        'status' => $status,
                        'created_by' => $userId
                    ]);
                 
            }
            
            return redirect()->route('fyugp.staff.student.attendance',['cdate'=>$validated['date']])->with(['message'=>'Attendance Added Successfully !','alert-type'=>'success']);
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
        
    }
    
    public function fyugp_details(Request $request) {
        
        $date = $request->date ?? null;
        $period_id = $request->period ?? null;
        $subject_id = $request->subject ?? null;
        $data_id = $request->dataid ?? null;
        $student_id = $request->sid ?? null;
        $course_id = $request->cid ?? null;
        $semester_id = $request->sem ?? null;
        
        if($date!=null && $period_id!=null && $subject_id!=null && $data_id!=null && $student_id!=null && $course_id!=null && $semester_id!=null) {
            $course = Course::find($course_id);
            $semester = Semester::find($semester_id);
            $subject = Subject::find($subject_id);
            $period = Periods::find($period_id);
            $student = Student::find($student_id);
            $timeperiod = FyugpTimetable::find($data_id);
            
            $attendance = StudentAttendance::where('date', $date)->where('course_id', $course_id)->where('semester_id', $semester_id)->where('subject_id', $subject_id)->where('period_id', $period_id)->where('student_id', $student_id)->first();
            $status=$reason='';
            if($attendance) {
                $status = $attendance->status;
                $reason = $attendance->reason;
            }
            
            return view('staff.fyugp.details',compact('course', 'semester', 'subject', 'period', 'date', 'student', 'attendance', 'status', 'reason', 'timeperiod'));
        }
        
        return false;
    }
    
    public function fyugp_leave(Request $request){
        try {
            $course = $request->course ?? null;
            $semester = $request->semester ?? null;
            $subject = $request->subject ?? null;
            $date = $request->date ?? null;
            $period = $request->period ?? null;
            $dataid = $request->dataid ?? null;
            $student = $request->student ?? null;
            $status = $request->status ?? null;
            $reason = $request->reason ?? null;
            $acyearId = getAcyear();
            
            if($course!=null && $semester!=null && $subject!=null && $date!=null && $period!=null && $dataid!=null && $student!=null && $status!=null && $acyearId!=null) {
                StudentAttendance::updateOrCreate([
                    'ac_id' => $acyearId,
                    'course_id' => $course,
                    'semester_id' => $semester,
                    'subject_id' => $subject,
                    'date' => $date,
                    'period_id' => $period,
                    'student_id' => $student
                ],[
                    'status' => $status,
                    'reason' => $reason
                ]);
                
                $msgArray = ['message'=>'Attendance Added Successfully !','alert-type'=>'success'];
            } else {
                $msgArray = ['message'=>'Attendance could not be Added !','alert-type'=>'error'];
            }
            
            return redirect()->route('fyugp.staff.student.attendance.add',['date'=>$date, 'period'=>$period, 'data_id'=>$dataid])->with($msgArray);
            
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: ' . $e->getMessage());
            return back()->with(['message'=>'Failed to load Data.','alert-type'=>'error']);
        }
    }
}