<?php

namespace App\Http\Controllers\Student;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

use App\Models\Progression;

class Progressions extends Controller
{
    public function report()
    {
        try {
            $student = Auth::guard('student')->user();
            $progression = Progression::where('student_id', $student->student_id)->first();
            
            return view('student.progression.report', compact('student', 'progression'));
        } catch (\Exception $e) {
            Log::error('Failed to fetch Progresstions: '.$e->getMessage());
            return back()->with(['message' => 'Failed to load Progresstions.', 'alert-type' => 'error']);
        }
    }
    
    public function save(Request $request)
    {
        $validated = $request->validate([
                        'student' => 'required',
                        'course' => 'required',
                        'adno' => 'nullable',
                        'passout' => 'nullable|numeric',
                        'cstatus' => 'required',
                        'contact' => 'nullable|numeric',
                        'email' => 'nullable|email',
                        'company' => 'nullable',
                        'location' => 'nullable',
                        'position' => 'nullable',
                        'join_date' => 'nullable|date',
                        'remarks' => 'nullable',
                    ]);
        try {
            $validated['date'] = date('Y-m-d');
            
            $saved = Progression::updateOrCreate([
                    'student_id' => Auth::guard('student')->id()
                ],$validated);
            
            if($saved){
                $msgArray = ['message' => 'Progresstion Saved Successfully.', 'alert-type' => 'success'];
            }else{
                $msgArray = ['message' => 'Progresstion could not be Saved.', 'alert-type' => 'error'];
            }
                
            return redirect()->route('student.progression')->with($msgArray);
                
        } catch (\Exception $e) {
            Log::error('Failed to save Progresstion: '.$e->getMessage());
            return back()->with(['message' => 'Failed to save Progresstion.', 'alert-type' => 'error']);
        }
    }
}
