<?php

namespace App\Http\Controllers\Student;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;

use App\Models\Periods;
use App\Models\Promote;
use App\Models\AcHoliday;
use App\Models\Timetable;
use App\Models\TimetableInterchange;
use App\Models\FyugpTimetable;
use App\Models\FyugpTimetableInterchange;

use DB;
use Carbon\Carbon;

class Timetables extends Controller
{
    public function report(Request $request)
    {
        try {
            $student = Auth::guard('student')->user();
            $studentId = $student->student_id;
            
            $promoteId = $request->input('promote') ?? null;
            $cdate = $request->input('cdate') ?? date('Y-m-d');
            $promotes = Promote::where('student_id', $studentId)->distinct('semester_id')->get();
            
            $timetables = $dates = $periods = $scodes = [];
            if($promoteId!=null && $cdate !=null)
            {
                $dayOfWeek = date('w', strtotime($cdate));
                if ($dayOfWeek == 0 || $dayOfWeek == 6)
                {
                    $lastFriday = date('Y-m-d', strtotime('last Friday'));
                    $cdate = $lastFriday;
                }
                $startOfWeek = Carbon::parse($cdate)->startOfWeek(Carbon::MONDAY);
                $endOfWeek = Carbon::parse($cdate)->endOfWeek(Carbon::FRIDAY);
                $currentDate = $startOfWeek;
                
                $promote = Promote::find($promoteId);
                $acyearId = $promote->ac_id;
                $periods = Periods::where('acyear', $acyearId)->get();
                
                if($student->ac_id < 6)
                {
                    $majorData = Timetable::where('course', $promote->course_id)->where('semester', $promote->semester_id)->where('acyear', $acyearId)->get();
                    $interchange = TimetableInterchange::whereBetween('date', [$startOfWeek->toDateString(), $endOfWeek->toDateString()])->get();
                    while ($currentDate->lte($endOfWeek))
                    {
                        $dateString = $currentDate->toDateString();
                        $dayName = $currentDate->format('l');
                        $dates[] = $dateString;
                        $holiday = AcHoliday::where('start_date', '<=', $dateString)->where('end_date', '>=', $dateString)->where('student', 1)->count();
                        
                        foreach ($periods as $period)
                        {
                            $majors = $majorData->where('day_of_week', $dayName)->where('period_id', $period->id);
                            if($majors)
                            {
                                foreach($majors as $major)
                                {
                                    $majorInter = $interchange->where('date', $dateString)->where('period_id', $period->id)->where('course_id', $promote->course_id)->where('semester_id', $promote->semester_id)->first();
                                    if($majorInter) {
                                        $timetables[$dateString][$period->id][] = [
                                                'category' => $major->subject->subcategoryname->category,
                                                'type' => 0,
                                                'subject' => $majorInter->subject->subject_name,
                                                'scode' => $majorInter->user->code,
                                                'holiday' => $holiday
                                            ];
                                        $scodes[$majorInter->user_id] = ['code' => $majorInter->user->code, 'name' => $majorInter->user->name];
                                    } else {
                                        $timetables[$dateString][$period->id][] = [
                                                'category' => $major->subject->subcategoryname->category,
                                                'type' => 0,
                                                'subject' => $major->subject->subject_name,
                                                'scode' => $major->unames->code,
                                                'holiday' => $holiday
                                            ];
                                        $scodes[$major->user_id] = ['code' => $major->unames->code, 'name' => $major->unames->name];
                                    }
                                }
                            }
                        }
                        $currentDate = $currentDate->addDay();
                    }
                } else {
                    $majorData = FyugpTimetable::where('course', $promote->course_id)->where('semester', $promote->semester_id)->where('categ_id', 1)->where('acyear', $acyearId)->get();
                    $interchange = FyugpTimetableInterchange::whereBetween('date', [$startOfWeek->toDateString(), $endOfWeek->toDateString()])->where('acyear_id', $acyearId)->get();
                    $groups = DB::table('group_courses')->where('course_id', $promote->course_id)->pluck('group_id')->toArray();
                    $subData = FyugpTimetable::join('fyugp_stud_sub', function($qry){
                            $qry->on('fyugp_timetable.categ_id', '=', 'fyugp_stud_sub.sub_cat')
                                ->on('fyugp_timetable.subject_id', '=', 'fyugp_stud_sub.sub_id');
                        })
                        ->where('fyugp_timetable.acyear', $acyearId)
                        ->where('fyugp_stud_sub.acyear_id', $acyearId)
                        ->where('fyugp_stud_sub.stud_id', $studentId)
                        ->where(function($qry2) use ($groups) {
                            $qry2->whereNull('fyugp_timetable.group_id')
                                ->orWhereIn('fyugp_timetable.group_id', $groups);
                        })
                        ->select('fyugp_timetable.*')
                        ->get();
                    
                    while ($currentDate->lte($endOfWeek))
                    {
                        $dateString = $currentDate->toDateString();
                        $dayName = $currentDate->format('l');
                        $dates[] = $dateString;
                        $holiday = AcHoliday::where('start_date', '<=', $dateString)->where('end_date', '>=', $dateString)->where('student', 1)->count();
                        
                        foreach ($periods as $period)
                        {
                            $majors = $majorData->where('day_of_week', $dayName)->where('period_id', $period->id);
                            if($majors)
                            { 
                                foreach($majors as $major)
                                {
                                    $majorInter = $interchange->where('date', $dateString)->where('period_id', $period->id)->where('course_id', $promote->course_id)->where('semester_id', $promote->semester_id)->where('category_id', 1)->first();
                                    if($majorInter) {
                                        $timetables[$dateString][$period->id][] = [
                                                'category' => $majorInter->category->category,
                                                'type' => $majorInter->type,
                                                'subject' => $majorInter->subject->subject_name,
                                                'scode' => $majorInter->user->code,
                                                'holiday' => $holiday
                                            ];
                                        $scodes[$majorInter->user_id] = ['code' => $majorInter->user->code, 'name' => $majorInter->user->name];
                                    } else {
                                        $timetables[$dateString][$period->id][] = [
                                                'category' => $major->subjectcateg->category,
                                                'type' => $major->type,
                                                'subject' => $major->subject->subject_name,
                                                'scode' => $major->user->code,
                                                'holiday' => $holiday
                                            ];
                                        $scodes[$major->user_id] = ['code' => $major->user->code, 'name' => $major->user->name];
                                    }
                                }
                            }
                            
                            $subs = $subData->where('day_of_week', $dayName)->where('period_id', $period->id);
                            if($subs)
                            {
                                foreach($subs as $sub)
                                {
                                    $timetables[$dateString][$period->id][] = [
                                            'category' => $sub->subjectcateg->category,
                                            'type' => $sub->type,
                                            'subject' => $sub->subject->subject_name,
                                            'scode' => $sub->user->code,
                                            'holiday' => $holiday
                                        ];
                                    $scodes[$sub->user_id] = ['code' => $sub->user->code, 'name' => $sub->user->name];
                                }
                            }
                        }
                        
                        $currentDate = $currentDate->addDay();
                    }
                }
            }
            
            return view('student.timetable.report', compact('promotes', 'promoteId', 'dates', 'periods', 'timetables', 'scodes', 'cdate'));
        } catch (\Exception $e) {
            Log::error('Failed to fetch Timetable : '.$e->getMessage());
            return back()->withErrors('Failed to load Timetable.');
        }
    }
}