<?php

namespace App\Http\Controllers\Student;

use App\Http\Controllers\Controller;
use App\Models\Student;
use App\Models\Acyear;
use App\Models\HostelFacility;
use App\Models\Religion;
use App\Models\acquotamodel;
use App\Models\FYUGPPATHWAY;
use App\Models\FYUGPSTUDPATHWAY;
use App\Models\FeeCategory;
use App\Models\Community;
use App\Models\Caste;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Image;

use DB;

class Users extends Controller
{
    public function login(){
        if(auth()->guard('student')->check()){
             return redirect()->route('student.home');
        }else{
            return view('student.login');
        }
    }
    
    public function logaction(){
        $input = ['email' => request('email'),'password' => request('password'),'status'=>1];
        if(auth()->guard('student')->attempt($input,true)){
            Cache::flush();
            return redirect()->route('student.home');
        }else{
            return redirect()->route('student.login')->with('message','Invalid Credentials or User Blocked !');
        }
    }
    
    public function logout(){
        auth()->guard('student')->logout();
        Cache::flush();
        return redirect()->route('student.login');
    }
    
    public function home(){
        
        $studentId = Auth::guard('student')->id();
        $semesters = DB::table('promotes')->join('semesters','promotes.semester_id','=','semesters.semester_id')->where('promotes.student_id',$studentId)->select('semesters.semester_id','semesters.semester_name')->groupBy('promotes.semester_id')->get();
        
        return view('student.home',compact('semesters'));
    }
    public function myprofile(){
        $studentId = Auth::guard('student')->id();
        $acyearId = getAcyear();
        $categories = FeeCategory::all();
        $acquota = acquotamodel::all();
        $religion=Religion::all();
        $student = Student::find($studentId);
        $community=Community::where('religion_id',$student->religion_id)->get(); 
        $castes=Caste::where('religion_id',$student->religion_id)->where('community_id',$student->community_id)->get(); 
        
        $promote = $student->promotes()->where('ac_id', $acyearId)->orderBy('promote_id', 'DESC')->first();
        $hostelfacility = HostelFacility::where('student_id',$studentId)->first();
        $pathway = FYUGPSTUDPATHWAY::join('fyugp_pathway','fyugp_stud_pathway.pathway_id','=','fyugp_pathway.id')->where('fyugp_stud_pathway.stud_id',$studentId)->where('fyugp_stud_pathway.course_id',$promote->course_id)->where('fyugp_stud_pathway.sem_id',$promote->semester_id)->where('fyugp_stud_pathway.ac_id',$acyearId)->first();
        return view('student.myprofile',compact('student', 'promote', 'hostelfacility', 'acyearId','acquota','categories','religion','pathway','community','castes'));
     }
     public function myprofile_update(Request $request){
          $studentId = Auth::guard('student')->id();
        $acyearId = getAcyear();
          $validated = $request->validate([
            'name' => 'required',
            'gender' => 'required',
            'mobile' => 'required|numeric',
            'mobile1' => 'nullable|numeric',
            'email'=>'required',
            'aadhar' => 'nullable|numeric',
            'dob' => 'required|date',
            'gmobile' => 'nullable|numeric',
            'fmobile' => 'nullable|numeric',
            'photo'=>'nullable|max:5120|mimes:jpeg,png,jpg',
            'income' => 'nullable|numeric',
            
           
        ]);
        $stud_det = Student::find($studentId);
         if($request->hasFile('photo')){
            $ext = $request->photo->extension();
            $photo = 'photo_'.time().'.'.$ext;
            
            Image::make($request->photo)->resize(300,300)->save('public/uploads/student/photo/'.$photo);
        }
        else
        {
           $photo =  $stud_det->photo;
        }
        $stud_det->update([
            'photo' =>$photo,'name'=>$request->name,'gender'=>$request->gender,'dob'=>$request->dob,'paddress'=>$request->paddress,'caddress'=>$request->caddress,'aadhar'=>$request->aadhar,'bgroup'=>$request->bgroup,
            'mobile'=>$request->mobile,'mobile1' =>$request->mobile1,'email'=>$request->email,'religion_id'=>$request->religion_id,'community_id'=>$request->community_id,'caste_id'=>$request->caste_id,'fname'=>$request->fname,'fmobile'=>$request->fmobile,'foccupation'=>$request->foccupation,
            'mname'=>$request->mname,'moccupation'=>$request->moccupation,'gname'=>$request->gname,'gaddress'=>$request->gaddress,'gmobile'=>$request->gmobile,'income'=>$request->income,'pline'=>$request->pline,'abc_id'=>$request->abc_id,
            'arts_sports'=>$request->arts_sports,'dscholar'=>$request->dscholar,'ncc_nss'=>$request->ncc_nss
            ]);
             if($stud_det){
            $msg_array = array('message'=>'Student Details Updated Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Student Details Could not be Updated !','alert-type'=>'error');
        }
        return redirect()->route('student.myprofile')->with($msg_array);
     }
}
