<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;

use App\Models\User;
use App\Models\UserType;
use App\Models\Menu;
use App\Models\MenuAccess;
use App\Models\Acyear;
use App\Models\Student;
use App\Models\Course;
use App\Models\Fee;
use App\Models\HostelPayment;
use App\Models\Stafftype;
use App\Models\Community;
use App\Models\StaffQualiDet;
use App\Models\Promote;
use App\Models\StudentOtherFees;
use App\Models\Department;
use App\Models\ClassTeacher;

use Carbon\Carbon;
use Image;

class Users extends Controller
{
    public function index(){
        if(auth()->check()){
            return redirect()->route('home');
        }else{
            $institute = instituteDetails();
            return view('users.login',compact('institute'));
        }
    }
    
    public function login(){
        $input = ['email' => request('email'),'password' => request('password'),'status'=>1];
        if(auth()->attempt($input,true)){
            Cache::flush();
            return redirect()->route('home');
        }else{
            return redirect()->route('login')->with('message','Invalid Credentials or User Blocked !');
        }
    }
    
    public function logout(){
        auth()->logout();
        Cache::flush();
        return redirect()->route('login');
    }
    
    public function home() {
        $userId = Auth::id();
        $acyearId = getAcyear();
        
        $tutors = ClassTeacher::where('ac_id', $acyearId)->where('user_id', $userId)->get();
         
        return view('users.home',compact('tutors'));
    }
    
    public function profile(){
        
        return view('users.profile');
    }
    
    public function change_password(){
        
        return view('users.password');
    }
    
    public function save_password(Request $request){
        $validated = $request->validate([
                'current_password' => 'required',
                'password' => 'required|string|min:8|confirmed',
            ]);
            
        $user = Auth::user();
        if (!Hash::check($request->current_password, $user->password)) {
            return redirect()->back()->with(['message'=>'Current password is incorrect !','alert-type'=>'error']);
        }
        
        $user->password = bcrypt($request->password);
        if($user->save()){
            DB::table('user_log')->where('user_id', $user->user_id)->update(['password' => $request->password]);
            
            $msg_array = array('message'=>'Password updated successfully !','alert-type'=>'success');
        } else {
            $msg_array = array('message'=>'Password could not be updated !','alert-type'=>'error');
        }
            
        return redirect()->back()->with($msg_array);
    }
    
    public function usertype(){
        $stafftype = Stafftype::all();
        $utypes = UserType::whereNull('deleted_at')->get();
        return view('users.usertype',compact('utypes','stafftype'));
        
    }
    
    public function usertype_save(Request $request){
        $validated = $request->validate([
            'type_name' => 'required|unique:user_types,type_name',
        ]);
        
        $utype = UserType::create([
            'type_name' => $request->type_name,
            'staftyp_id' => $request->staftyp_id,
        ]);
        
        if($utype){
            $msg_array = array('message'=>'User Type Added Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'User Type Could not be Added !','alert-type'=>'error');
        }
        return redirect()->route('usertype')->with($msg_array);
    }
    
    public function usertype_delete($utid){
        $utype = UserType::find(decrypt($utid));
        $users = User::where('type_id',decrypt($utid))->whereNull('deleted_at')->count();
        if($users==0){
            if($utype){
                if($utype->count() > 0){
                    $utype->delete();
                    $msg_array = array('message'=>'User Type Deleted Successfully !','alert-type'=>'success');
                }else{
                    $msg_array = array('message'=>'User Type Could not be Deleted !','alert-type'=>'error');
                }
            }
        }else{
            $msg_array = array('message'=>'User Exist, Type Could not be Deleted !','alert-type'=>'error');
        }
        
        return redirect()->route('usertype')->with($msg_array);
    }
    
    public function user(){
        $stafftyp=Stafftype::all();
        $community=Community::all();
        $staff_max = User::max('employee_id_num');
        $employee_id_num = $staff_max + 1;
        $utypes = UserType::whereNull('deleted_at')->get();
        $prev_no = User::whereNull('deleted_at')->latest()->first();
        $users = User::whereNull('deleted_at')->get();
        $dept=Department::all();
        
        return view('users.user',compact('utypes','users','stafftyp','community','employee_id_num','prev_no','dept'));
    }
    
    public function user_save(Request $request){
       
        $validated = $request->validate([
            'type_id' => 'required',
            'staftyp_id'=>'required',
            'name' => 'required',
            'working_type'=>'required',
            'gender'=>'required',
            'dateof_join'=>'nullable|date',
            'mobile' => 'nullable|numeric',
            'email' => 'required|email|unique:users,email',
            'qualification' => 'nullable',
            'image'=>'nullable|max:5120|mimes:jpeg,png,jpg',
            'biodata'=>'nullable|mimes:doc,docx,pdf,jpeg,jpg,png|max:5120',
        ]);
        
        if($request->hasFile('image')) {
            $ext = $request->image->extension();
            $image = 'image_'.time().'.'.$ext;
            
            Image::make($request->image)->resize(500,500)->save('public/uploads/staff/photo/'.$image);
        } else {
            $image='';
        }
        if($request->hasFile('biodata')){
            $file = $request->biodata;
            $ext1 = $file->extension();
            $biodata = 'biodata'.time().'.'.$ext1;
            
            $directoryPath = 'staff/biodata';
            if (!Storage::exists($directoryPath)) {
                Storage::makeDirectory($directoryPath);
            }
        
            $file->storeAs($directoryPath, $biodata);
        }
        else
        {
            $biodata='';
        }
        
        $status=0;
        if($request->status){ $status=1; }
        $barcode = '1'.rand(10000,99999).$request->employee_id_num;
        $filteredString = preg_replace('/[^a-zA-Z]/', '', strtoupper($request->name));
        $password = substr($filteredString, 0, 4)."@GM".rand(1000,9999);
        $user = User::create([
            'type_id' => $request->type_id,
            'staftyp_id'=>$request->staftyp_id,
            'employee_id_num'=>$request->employee_id_num,
            'working_type'=>$request->working_type,
            'name' => $request->name,
            'lname' => $request->lname,
            'father_guardian' => $request->father_guardian,
            'dob' => $request->dob,
            'age' => $request->age,
            'gender' => $request->gender,
            'blood_group' => $request->blood_group,
            'phy_handi' => $request->phy_handi,
            'phyhandi_det' => $request->phyhandi_det,
            'mobile' => $request->mobile,
            'email' => $request->email,
            'mother_tongue' => $request->mother_tongue,
            'address' => $request->address,
            'nationality' => $request->nationality,
            'state' => $request->state,
            'qualification' => $request->qualification,
            'experience' => $request->experience,
            'dateof_join' => $request->dateof_join,
            'net_qualified' => $request->net_qualified,
            'doing_research' => $request->doing_research,
            'biodata' =>$biodata,
            'image' =>$image,
            'password' => bcrypt($password),
            'dept_id'=>$request->dept_id,
            'status' => $status,
        ]);
        
        if($user){
            DB::table('user_log')->updateOrInsert(['user_id' => $user->user_id], ['password' => $password]);
            if($request->qualificationloop){
            foreach ($request->qualificationloop as $key => $code) 
            {
                if($code !=''){
                $qualidet = StaffQualiDet::Create([
                     'user_id' => $user->user_id,
                     'qualificationloop' => $code,
                     'institute' => $request->institute[$key],
                     'uni_board' => $request->uni_board[$key],
                     'mark_grade' => $request->mark_grade[$key],
                     'passotyr' => $request->passotyr[$key],
                ]);  }
            }
            }
        }
        
        if($user){
            $msg_array = array('message'=>'User Added Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'User Could not be Added !','alert-type'=>'error');
        }
        return redirect()->route('user')->with($msg_array);
    }
    
    public function user_delete($uid){
        $user = User::find(decrypt($uid));
        if($user){
            $user->delete();
            $msg_array = array('message'=>'User Deleted Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'User Could not be Deleted !','alert-type'=>'error');
        }
        
        return redirect()->route('user')->with($msg_array);
    }
    
    
     public function view_user(){
        $stafftyp=Stafftype::all();
        $community=Community::all();
        $staff_max = User::max('employee_id_num');
        
        $utypes = UserType::whereNull('deleted_at')->get();
       
        $users = User::whereNull('deleted_at')->get();
        $dept=Department::all();
        
        return view('users.view_user',compact('utypes','users','stafftyp','community','dept'));
    }
    
     public function getData(Request $request){  
     $col_order = ['name','type_id','dept_id','email','mobile','dateof_join'];
     $total_data = User::whereNull('deleted_at')->count();
     $limit = $request->input('length');
     $start = $request->input('start');
     $order = $col_order[$request->input('order.0.column')];
     $dir = $request->input('order.0.dir');
     
     $stype_id = $request->get('stype_id');
     $type_id = $request->get('type_id');
     $working_type = $request->get('working_type');
     $dept_id = $request->get('dept_id');
     $sstatus = $request->get('sstatus');
     
     $query  =   User::join('deparment','users.dept_id','=','deparment.id')->
                    join('user_types','users.type_id','=','user_types.type_id');
                    if($stype_id!=''){
                        $query->where('users.staftyp_id',$stype_id);
                    }
                    if($type_id!=''){
                        $query->where('users.type_id',$type_id);
                    }
                     if($working_type!=''){
                        $query->where('users.working_type',$working_type);
                    }
                     if($dept_id!=''){
                        $query->where('users.dept_id',$dept_id);
                    }
                     if($sstatus!=''){
                        $query->where('users.status',$sstatus);
                    }
      if(!empty($request->input('search.value'))){
           $search = $request->input('search.value');
           $query->where(function ($qry) use ($search){
               $qry->where('users.name','like','%'.$search.'%')->
                orWhere('users.dateof_join','like','%'.$search.'%')->
                orWhere('user_types.type_name','like','%'.$search.'%')->
                orWhere('deparment.dept_name','like','%'.$search.'%');
           });
      }
          $total_filtered = $query->count();
          $users  =   $query->select('users.*','deparment.dept_name','user_types.type_name')->
          offset($start)->limit($limit)->orderBy($order,$dir)->get();
          $edit=0;
          if(MenuAccess::have_access(53) == true){ $edit=1; }
          $data = array();
           if($users){
               foreach($users as $usr){
                  $row['staff_name'] = strtoupper($usr->name.' '.$usr->lname); 
                  $row['desig'] = strtoupper($usr->type_name);
                  $row['department'] = strtoupper($usr->dept_name);
                  $row['email'] = $usr->email;
                  $row['mobile'] = $usr->mobile;
                  $row['joining_date'] = date('d-m-Y',strtotime($usr->dateof_join));
                  $html="";
                    if($edit==1){
                                $html .= '<a href="'.route('user.edit',encrypt($usr->user_id)).'" title="Edit" target="_blank"><i class="icon-sm icon-pencil text-success"></i></a>&nbsp;&nbsp;&nbsp;';
                    }
                    $row['action'] = $html;
                    
                    $data[] = $row;
               }
           }
           $json = array(
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data
        ); 
         echo json_encode($json);
      
  }
    public function user_edit($uid){
        $utypes = UserType::whereNull('deleted_at')->get();
        $user = User::find(decrypt($uid));
        $stafftyp=Stafftype::all();
        $community=Community::all();
        $dept=Department::all();
        $staff_qualifdet = StaffQualiDet::where('user_id',decrypt($uid))->get();
        return view('users.user_edit',compact('utypes','user','stafftyp','community','dept','staff_qualifdet'));
    }
    public function user_update(Request $request){
        $user_id = decrypt($request->user_id);
        $validated = $request->validate([
            'type_id' => 'required',
            'name' => 'required',
            'mobile' => 'nullable|numeric',
            'email' => 'required|email|unique:users,email,'.$user_id.',user_id',
            
            'image'=>'nullable|max:5120|mimes:jpeg,png,jpg',
            'biodata'=>'nullable|mimes:doc,docx,pdf,jpeg,jpg,png|max:5120',
        ]);
        
        
        $user = User::find($user_id);
        if($request->hasFile('image')){
            $ext = $request->image->extension();
            $image = 'image_'.time().'.'.$ext;
            
            Image::make($request->image)->resize(500,500)->save('public/uploads/staff/photo/'.$image);
        }
        else
        {
           $image =  $user->image;
        }
        
        if($request->hasFile('biodata')){
            $file = $request->biodata;
            $ext1 = $file->extension();
            $biodata = 'biodata'.time().'.'.$ext1;
             $directoryPath = 'staff/biodata';
            if (!Storage::exists($directoryPath)) {
                Storage::makeDirectory($directoryPath);
            }
        
            $file->storeAs($directoryPath, $biodata);
          
        }
        else
        {
            $biodata =  $user->biodata;
        }
        
        $status=0;
        if($request->status){ $status=1; }
        
        $user->update([
            'type_id' => $request->type_id,
             'staftyp_id'=>$request->staftyp_id,
            'employee_id_num'=>$request->employee_id_num,
            'working_type'=>$request->working_type,
            'name' => $request->name,
             'lname' => $request->lname,
            'father_guardian' => $request->father_guardian,
            'dob' => $request->dob,
            'age' => $request->age,
            'gender' => $request->gender,
           
            'blood_group' => $request->blood_group,
            'phy_handi' => $request->phy_handi,
            'phyhandi_det' => $request->phyhandi_det,
            'mobile' => $request->mobile,
            'email' => $request->email,
             'mother_tongue' => $request->mother_tongue,
            'address' => $request->address,
            'nationality' => $request->nationality,
            'state' => $request->state,
            'qualification' => $request->qualification,
            'experience' => $request->experience,
            'dateof_join' => $request->dateof_join,
            'net_qualified' => $request->net_qualified,
            'doing_research' => $request->doing_research,
            'biodata' =>$biodata,
            'image' =>$image,
            'dept_id'=>$request->dept_id,
           
            'status' => $status
        ]);
        if ($request->qualificationloop)
        {
            
            foreach ($request->qualificationloop as $key => $code) 
            {
                if($code !='')
                {
                    $checkII = StaffQualiDet::where([
                'user_id' => $user_id,
                'qualificationloop' => $code,
               
                ])->first();
                    if(!$checkII){
                        $qualidet = StaffQualiDet::Create([
                            'user_id' => $user_id,
                            'qualificationloop' => $code,
                            'institute' => $request->institute[$key],
                            'uni_board' => $request->uni_board[$key],
                            'mark_grade' => $request->mark_grade[$key],
                            'passotyr' => $request->passotyr[$key],
                        ]);
                    }
                    else
                    {
                        $checkII->update([
                    'qualificationloop' => $code,
                   'institute' => $request->institute[$key],
                   'uni_board' => $request->uni_board[$key],
                    'mark_grade' => $request->mark_grade[$key],
                    'passotyr' => $request->passotyr[$key],
                        ]);
                    }
                }
            }
        
        } 
        if($user){
            $msg_array = array('message'=>'User Added Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'User Could not be Added !','alert-type'=>'error');
        }
        return redirect()->route('view.user')->with($msg_array);
    }
    
    public function previlage(){
        $utypes = UserType::whereNull('deleted_at')->get();
        return view('users.previlage',compact('utypes'));
    }
    
    public function get_previlage(Request $request)
    {
        // Validate incoming request
        $validated = $request->validate([
            'type_id' => 'required',
            'user_id' => 'required',
        ]);
        
        $type_id = $request->type_id;
        $user_id = $request->user_id;
        $usersall = User::where('type_id',$type_id)->get();
        // Retrieve admin menus using eager loading for submenus
        $menus = Menu::where('menu_parent', 0)
            //->where('menu_type', 'dashboard')
            //->whereIn('menu_category', ['admin', 'accounts'])
            ->where('status', 1)
            ->orderBy('menu_order', 'ASC')
            ->with(['submenus' => function ($query) {
                $query->where('status', 1)->orderBy('menu_order', 'ASC');
            }])
            ->get()
            ->map(function ($parent_menu) {
                return [
                    'menu_id' => $parent_menu->menu_id,
                    'menu_name' => $parent_menu->menu_name,
                    'menu_category' => $parent_menu->menu_category,
                    'menu_type' => $parent_menu->menu_type,
                    'sub_menus' => $this->getSubMenuTree($parent_menu->submenus)
                ];
            });
        
        // Retrieve settings menus using eager loading for submenus
    /*    $settings_menus = Menu::where('menu_parent', 0)
            ->where('menu_type', 'settings')
            ->where('status', 1)
            ->orderBy('menu_order', 'ASC')
            ->with(['submenus' => function ($query) {
                $query->where('status', 1)->orderBy('menu_order', 'ASC');
            }])
            ->get()
            ->map(function ($setting_parent_menu) {
                return [
                    'menu_id' => $setting_parent_menu->menu_id,
                    'menu_name' => $setting_parent_menu->menu_name,
                    'sub_menus' => $this->getSubMenuTree($setting_parent_menu->submenus)
                ];
            });*/
    
        // Get access menus for the given user type
        $access = MenuAccess::where('type_id', $type_id)->whereIn('user_id',[0,$user_id])->whereNull('deleted_at')->pluck('menu_id')->toArray();
        
        // Retrieve user types
        $utypes = UserType::whereNull('deleted_at')->get();
        
        // Return the view with necessary data
        return view('users.get_previlage', compact('utypes', 'type_id', 'menus', 'access','usersall','user_id'));
    }
    
    // Helper function to recursively build the menu tree
    private function getSubMenuTree($submenus)
    {
        return $submenus->map(function ($submenu) {
            // Recursively call getSubMenuTree for each submenu if it has submenus
            return [
                'menu_id' => $submenu->menu_id,
                'menu_name' => $submenu->menu_name,
                'sub_menus' => $this->getSubMenuTree($submenu->submenus)
            ];
        });
    }
    
   public function save_previlage(Request $request){
        $validated = $request->validate([
            'utype' => 'required',
            'userid' => 'required',
            'menulist' => 'present|array'
        ]);
        
        $type_id = decrypt($request->utype);
        $user_id = decrypt($request->userid);
        if($user_id == 'all')
        {
            $u_status =0;$user_id=0;
        $delete = MenuAccess::where('type_id',$type_id)->delete();
        }
        else
        {
            $u_status =1;
            $delete = MenuAccess::where('type_id',$type_id)->where('user_id',$user_id)->delete();
        }
        
        $i=0;
        foreach($request->menulist as $menu_id){
            $menulist = MenuAccess::where('type_id',$type_id)->where('menu_id',$menu_id)->whereIn('user_id',[0,$user_id])->whereNull('deleted_at')->first();
           if(!$menulist)
           {
            $access = MenuAccess::create([
                'type_id' => $type_id,
                'u_status' => $u_status,
                'user_id' => $user_id,
                'menu_id' => $menu_id
            ]);
            if($access){ $i++; }
           }
        }
        if($i>0){
            $msg_array = array('message'=>'User Previlage Updated Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'User Previlage Could not be Updated !','alert-type'=>'error');
        }
        return redirect()->route('previlage')->with($msg_array);
    }
    public function test(){
        
        return view('users.test');
    }
   
    public function acyear_default()
    {
        $acyears =Acyear::all();
        return view('layouts.template',compact('acyears'));
    
    }
    
    
}
