<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Auth;
use App\Models\Semester;
use App\Models\StaffSubject;
class Subject extends Model
{
    use HasFactory,SoftDeletes;
    
    protected $table = 'subjects';
    
    protected $primaryKey = 'subject_id'; 
    
    protected $guarded = [];
    
    protected $hidden = ['subject_id'];
    
    public function submax(){
        return $this->hasmany(ObeSubMaxMark::class,'sub_id','subject_id');
    }
    
    public function marklists(){
        return $this->hasmany(ObeMarkList::class,'sub_id','subject_id');
    }
    
     public function subco(){
        return $this->hasmany(ObeCo::class,'sub_id','subject_id');
    }
    public function SubtypeName(){
        return $this->belongsTo(SubjectType::class,'subtyp_id','subtyp_id')->withDefault(['name' => '']);
    }
    
    public function subcredits(){
        return $this->hasmany(FyugpSubjectCredit::class,'subject','subject_id');
    }
     public function course(){
        return $this->belongsTo(Course::class,'course_id','course_id')->withDefault(['name' => '']);
    }
     public function sem(){
        return $this->belongsTo(Semester::class,'semester_id','semester_id')->withDefault(['name' => '']);
    }
     /*public function subtype(){
        return $this->hasOne(SubjectType::class,'subtyp_id','subject_categ');
    }*/
    
     public function subcategoryname(){
        return $this->belongsTo(SubCategory::class,'subject_categ','id')->withDefault(['name' => '']);
    }
     
   public function minsubstudtotcnt(){
        return $this->hasmany(FyugpStudSub::class,'sub_id','subject_id');
    }
     public function coursesubjects()
    {
        return $this->hasMany(CourseSubject::class, 'subject_id', 'subject_id');
    }
     public function deptName(){
        return $this->belongsTo(Department::class,'department_id','id')->withDefault(['name' => '']);
    }
    
     public static function getStaffSubjects($semesterId) {
        $user = Auth::user();
        $acyearId = getAcyear();
        
        $semester = Semester::find($semesterId);
        $query = self::query();
        
        if(in_array($user->type_id, [12,20,21,36])) {
            $subs = StaffSubject::where('user_id', $user->user_id)->where('acyear_id', $acyearId)->where('course_id', $semester->course_id)->where('semester_id', $semesterId)->pluck('subject_id')->toArray();
            $query->whereIn('subject_id', $subs);
        } else {
           if($user->dept_id ==9)
            {
            $subs = StaffSubject::where('user_id', $user->user_id)->where('acyear_id', $acyearId)->where('course_id', $semester->course_id)->where('semester_id', $semesterId)->pluck('subject_id')->toArray();
            $query->whereIn('subject_id', $subs);
            }
            else
            {
               $subs = StaffSubject::where('acyear_id', $acyearId)->where('course_id', $semester->course_id)->where('semester_id', $semesterId)->pluck('subject_id')->toArray();
               $query->whereIn('subject_id', $subs);  
            }
        }
        
        $subjects = $query->groupBy('subject_id')->orderBy('subject_categ','asc')->get();
        
        return $subjects;
    }
    
    public function subcreditsstd(){
        return $this->hasmany(FyugpSubjectCredit::class,'subject','subject_id');
    }
    
     public function exmarksstd(){
        return $this->hasmany(STUDEXMMARKS::class,'subject','subject_id');
    }
  
    public function examSubjects()
    {
        return $this->hasOne(FyugpExamSubject::class, 'subject', 'subject_id');
    }
}
