@extends('layouts.template')
@section('title','Generate FYUGP Time Table')
@section('heading','Generate FYUGP Time Table')
@section('breadcrumb','Home / FYUGP Time Table / Generate')
@section('content')
    <style>
        .timetable {
            text-transform: uppercase;
            text-align: center;
            align-items: center;
            font-size: 13px;
        }
        .tt-row{
            display: flex;
            gap: 10px;
        }
        .tt-block {
            flex: 1 1 calc(100% / 7);
        }
        .tt-period {
            background-color: #2D2870;
            color: #FFF;
            font-weight: 700;
            padding: 10px 15px;
            border-radius: 10px;
            margin: 5px 0;
        }
        .tt-day {
            background-color: #DC4F4B;
            color: #FFF;
            font-weight: 700;
            padding: 10px 15px;
            border-radius: 10px;
            margin: 5px 0;
            height: auto;
            min-height: 180px;
            align-items: center;
        }
        .tt-content {
            background-color: #F1FAEE;
            border-radius: 10px;
            margin: 5px 0;
            padding: 5px;
            height: auto;
            min-height: 180px;
        }
        .tt-item {
            background-color: #c7e9ff;
            border-radius: 5px;
            margin: 3px 0;
        }
        .tt-item:hover {
            cursor: pointer;
        }
        .tt-category {
            font-weight: 600;
        }
        .tt-user {
            
        }
        .tt-button {
            background-color: green;
            color: #FFF;
            padding: 2px 10px;
            border: 0px;
            border-radius: 5px;
        }
        
        @media (max-width: 480px) {
            .tt-row{
                gap: 5px;
            }
            .tt-block {
                flex: 1 1 100%;
            }
        }
    </style>
    <div class="card">
        <div class="card-body">
            <form method="get">
                <div class="row">
                    <div class="col-12 col-md-3">
                        <div class="form-group">
                            <label for="course" class="required">Programme</label>
                            <select class="form-control" id="course" name="course" required>
                                <option value="">Select</option>
                                @foreach($courses as $coursedet)
                                    <option value="{{ $coursedet->course_id }}" {{ $coursedet->course_id == $courseId ? 'selected':'' }}>{{ $coursedet->course_name }}</option>
                                @endforeach
                            </select>
                            @error('course')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                        </div>
                    </div>
                    <div class="col-12 col-md-6">
                        <div class="form-group">
                            <label for="semester" class="required">Semester</label>
                            <div class="row">
                                <div class="col-12 col-md-6">
                                    <div class="form-group">
                                        <select class="form-control" id="semester" name="semester" required>
                                            <option value="">Select</option>
                                            @foreach($semesters as $semesterdet)
                                                <option value="{{ $semesterdet->semester_id }}" {{ $semesterdet->semester_id==$semesterId ? "selected":"" }}>{{ $semesterdet->semester_name }}</option>
                                            @endforeach
                                        </select>
                                    </div>
                                </div>
                                <div class="col-12 col-md-6">
                                    <div class="form-group">
                                        <button type="submit" class="btn btn-sm btn-success">Submit</button>
                                    </div>                   
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </form>
        </div>
    </div>
    @if(!empty($data))
        <br>
        <div class="card">
            <div class="card-body">
                <div class="row">
                    <div class="col-12 text-center">
                        <h4>{{ $course ? $course->course_name : '' }}</h4>
                    </div>
                </div>
                <div class="timetable">
                    <div class="tt-row">
                        <div class="tt-block">
                            <div class="tt-head"></div>
                        </div>
                        @foreach($periods as $period)
                            <div class="tt-block">
                                <div class="tt-period">{{ $period->period }}</div>
                            </div>
                        @endforeach
                    </div>
                    @foreach($days as $day)
                        <div class="tt-row">
                            <div class="tt-block">
                                <div class="tt-day">{{ $day->day }}</div>
                            </div>
                            @foreach($periods as $period)
                                <div class="tt-block">
                                    <div class="tt-content">
                                        @if(!empty($data[$day->day][$period->id]))
                                            @foreach($data[$day->day][$period->id] as $category)
                                                <div class="tt-item" onclick="editSlot('{{ $courseId }}','{{ $semesterId }}','{{ $day->day }}','{{ $period->id }}','{{ $category['data_id'] }}');">
                                                    <div class="tt-category">{{ $category['category_name'] }}</div>
                                                    <div class="tt-user">
                                                        @if(in_array($category['category_id'], [5,6]))
                                                            {{ $category['group_name'].' - ' }}
                                                        @endif
                                                        {{ $category['user_code'] }}
                                                    </div>
                                                </div>
                                            @endforeach
                                        @endif
                                        <div>
                                            <button type="button" class="tt-button" onclick="addSlot('{{ $courseId }}','{{ $semesterId }}','{{ $day->day }}','{{ $period->id }}')">Add</button>
                                        </div>
                                    </div>
                                </div>
                            @endforeach
                        </div>
                    @endforeach
                </div>
                <br>
                
                <div class="row">
                    @foreach($staffs as $staff)
                        <div class="col-12 col-sm-4">
                            <div class="form-group">
                                <span class="text-danger">{{ $staff->code }} : </span>{{ $staff->name }}
                            </div>
                        </div>
                    @endforeach
                </div>
            </div>
        </div>
    @endif
    <div class="modal fade" id="timetableModal" tabindex="-1" role="dialog" aria-labelledby="timetableModalLabel" aria-hidden="true">
        <div class="modal-dialog" role="document">
            <div class="modal-content">
                <div class="modal-header">
                    <h5 class="modal-title" id="timetableModalLongTitle">Timetable</h5>
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
                <div class="modal-body">
                </div>
            </div>
        </div>
    </div>
    <script src="{{ asset('public/js/fyugp/timetable.js?ver=1.69') }}"></script>
    <script>
        function editSlot(cid, sid, day, pid, id){
            var baseUrl = window.location.origin;
            if(cid!='' && sid!='' && day!='' && pid!='' && id!=''){
                $.ajax({
                    type : 'GET',
                    url : baseUrl+'/generate-fyugptimetable-edit',
                    data : { cid: cid, sid: sid, day: day, pid: pid, id: id },
                    success : function(response){
                        $('.modal-body').html(response);
                        $('#timetableModal').modal('show');
                    },
                    error: function (xhr, status, error) {
                        alert('Could not able to fetch data. Try again later !');
                    }
                });
            }else{
                alert('Invalid Data');
            }
        }
        
        function addSlot(cid, sid, day, pid){
            var baseUrl = window.location.origin;
            if(cid!='' && sid!='' && day!='' && pid!=''){
                $.ajax({
                    type : 'GET',
                    url : baseUrl+'/generate-fyugptimetable-add',
                    data : { cid: cid, sid: sid, day: day, pid: pid },
                    success : function(response){
                        $('.modal-body').html(response);
                        $('#timetableModal').modal('show');
                    },
                    error: function (xhr, status, error) {
                        alert('Could not able to fetch data. Try again later !');
                    }
                });
            }else{
                alert('Invalid Data');
            }
        }
    </script>
@endsection