@extends('layouts.template')
@section('title','Membership')
@section('heading','New Student Membership')
@section('breadcrumb','Home / Library / Membership / Student / Add')
@section('content')
    <div class="card">
        <div class="card-body">
            <div class="row">
                <div class="col-12 col-md-4">
                    <div class="form-group">
                        <label for="course">Program</label>
                        <select class="js-example-basic-single form-control" id="course">
                            <option value="">Select</option>
                            @foreach($courses as $course)
                                <option value="{{ $course->course_id }}">{{ $course->course_name }}</option>
                            @endforeach
                        </select>
                    </div>
                </div>
                <div class="col-12 col-md-8">
                    <div class="form-group">
                        <label for="semester">Semester</label>
                        <div class="row">
                            <div class="col-12 col-md-6">
                                <div class="form-group">
                                    <select class="js-example-basic-single form-control" id="semester">
                                        <option value="">Select</option>
                                    </select>
                                </div>
                            </div>
                            <div class="col-12 col-md-6">
                                <div class="form-group">
                                    <button type="button" class="btn btn-sm btn-success" id="filter">Filter</button>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
    <br>
    <div class="card">
        <div class="card-body">
            <form action="{{ route('library.membership.student.save') }}" method="post">
                @csrf
                <div class="row">
                    <div class="col-12 col-md-6">
                        <div class="form-group">
                            <label for="date">Date</label>
                            <div class="row">
                                <div class="col-12 col-md-6">
                                    <div class="form-group">
                                        <input type="date" class="form-control @error('date') border-danger @enderror" id="date" name="date" value="{{ old('date') ?? date('Y-m-d') }}" required>
                                        @error('date')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                                    </div>
                                </div>
                                <div class="col-12 col-md-6">
                                    <div class="form-group">
                                        <button type="submit" class="btn btn-sm btn-success" id="filter">Save</button>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
                <table class="table table-bordered" id="studentTable" width="100%">
                    <thead>
                    <tr>
                        <th class="no-sort">
                            <div class="form-check">
                                <input type="checkbox" id="checkAll">
                            </div>
                        </th>
                        <th>Adm No.</th>
                        <th>Student Name</th>
                        <th>Program</th>
                        <th>Semester</th>
                        <th>Contact No</th>
                    </tr>
                    </thead>
                    <tbody>
                    </tbody>
                </table>
            </form>
        </div>
    </div>
    <script src="{{ asset('public/js/library/membership.js') }}"></script>
    <script>
        $(function () {
            var studentTable = $("#studentTable").DataTable({
                lengthChange: true,
                lengthMenu: [ [10, 50, 100, 500, -1], [10, 50, 100, 500, 'All'],],
                scrollX: true,
                searching: true,
                processing : true,
                serverSide : true,
                stateSave : true,
                columnDefs : [
                    { targets: 'no-sort', orderable: false },
                    { targets: [0, 1, 3, 4, 5], className: 'text-center' }
                ],
                ajax : {
                    url : '{{ route('library.membership.student.list') }}',
                    dataType : 'json',
                    type : 'GET',
                    data: function(data){
                        data.course = $('#course').val(),
                        data.semester = $('#semester').val();
                    }
                },
                columns : [
                    { data : 'check' },
                    { data : 'adno' },
                    { data : 'name' },
                    { data : 'course' },
                    { data : 'semester' },
                    { data : 'contact' },
                ]
            });
            
            $('#filter').click(function(){
                studentTable.draw();
            });
        });
    </script>
@endsection