@extends('layouts.student-template')
@section('title','Attendance')
@section('heading','Attendance Report')
@section('breadcrumb','Home / Timetable')
@section('content')
    <style>
        .timetable {
            text-transform: uppercase;
            text-align: center;
            align-items: center;
            font-size: 13px;
        }
        .tt-row{
            display: flex;
            gap: 10px;
        }
        .tt-block {
            flex: 1 1 calc(100% / 7);
        }
        .tt-period {
            background-color: #2D2870;
            color: #FFF;
            font-weight: 700;
            padding: 10px 15px;
            border-radius: 10px;
            margin: 5px 0;
        }
        .tt-day {
            background-color: #DC4F4B;
            color: #FFF;
            font-weight: 700;
            padding: 10px 15px;
            border-radius: 10px;
            margin: 5px 0;
            height: auto;
            min-height: 180px;
            align-items: center;
        }
        .tt-content {
            background-color: #F1FAEE;
            border-radius: 10px;
            margin: 5px 0;
            padding: 5px;
            height: auto;
            min-height: 180px;
        }
        .tt-item {
            background-color: #c7e9ff;
            border-radius: 5px;
            margin: 3px 0;
        }
        .tt-item:hover {
            cursor: pointer;
        }
        .tt-category {
            font-weight: 600;
        }
        .tt-user {
            
        }
        .tt-button {
            background-color: green;
            color: #FFF;
            padding: 2px 10px;
            border: 0px;
            border-radius: 5px;
        }
        
        @media (max-width: 480px) {
            .tt-row{
                gap: 5px;
            }
            .tt-block {
                flex: 1 1 100%;
            }
        }
    </style>
    <div class="card">
        <div class="card-body">
            <form method="get">
                <div class="row">
                    <div class="col-12 col-sm-3">
                        <div class="form-group">
                            <label for="promote">Semester</label>
                            <select class="form-control" id="promote" name="promote">
                                <option value="">Select</option>
                                @foreach($promotes as $promote)
                                    <option value="{{ $promote->promote_id }}" {{ $promote->promote_id == $promoteId ? 'selected':'' }}>{{ $promote->semester->semester_name }}</option>
                                @endforeach
                            </select>
                            @error('promote')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                        </div>
                    </div>
                    <div class="col-12 col-sm-6">
                        <div class="form-group">
                            <label for="cdate">Date</label>
                            <div class="row">
                                <div class="col-12 col-sm-6">
                                    <div class="form-group">
                                        <input type="date" class="form-control @error('cdate') border-danger @enderror" id="cdate" name="cdate" value="{{ $cdate }}">
                                        @error('cdate')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                                    </div>
                                </div>
                                <div class="col-12 col-sm-6">
                                    <div class="form-group">
                                        <button type="submit" class="btn btn-sm btn-success">Submit</button>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </form>
        </div>
    </div>
    <br>
    @if(!empty($timetables))
        <div class="card">
            <div class="card-body">
                <div class="timetable">
                    <div class="tt-row">
                        <div class="tt-block">
                            <div class="tt-head"></div>
                        </div>
                        @foreach($periods as $period)
                            <div class="tt-block">
                                <div class="tt-period">{{ $period->period }}</div>
                            </div>
                        @endforeach
                    </div>
                    @foreach($dates as $date)
                        <div class="tt-row">
                            <div class="tt-block">
                                <div class="tt-day">
                                    <div>{{ $date }}</div>
                                    <div>{{ date('l', strtotime($date)) }}</div>
                                </div>
                            </div>
                            @foreach($periods as $period)
                                <div class="tt-block">
                                    <div class="tt-content">
                                        @if(isset($timetables[$date][$period->id]))
                                            @foreach($timetables[$date][$period->id] as $timetable)
                                                <div class="tt-item">
                                                    <div class="tt-category">{{ $timetable['category'] }} @if($timetable['type']==1) {{ ' - PR' }} @endif</div>
                                                    <div class="tt-user text-success">{{ $timetable['subject'] }}</div>
                                                    <div class="tt-user">{{ $timetable['scode'] }}</div>
                                                </div>
                                            @endforeach
                                        @endif
                                    </div>
                                </div>
                            @endforeach
                        </div>
                    @endforeach
                </div>
                <br>
                <div class="row">
                    @foreach($scodes as $scode)
                        <div class="col-12 col-sm-4">
                            <div class="form-group">
                                <span class="text-danger">{{ strtoupper($scode['code']) }} : </span>{{ strtoupper($scode['name']) }}
                            </div>
                        </div>
                    @endforeach
                </div>
            </div>
        </div>
    @else
        <div class="alert alert-danger">No Result Found !</div>
    @endif
@endsection