@extends('layouts.template')
@section('title','Timetable')
@section('heading','Timetable')
@section('breadcrumb','Home / Timetable / Interchange')
@section('content')
    <style>
        .timetable {
            text-transform: uppercase;
            text-align: center;
            align-items: center;
            font-size: 13px;
            margin-top: 15px;
        }
        .tt-row{
            display: flex;
            gap: 10px;
        }
        .tt-head {
            font-weight: 600;
            color: #FFF;
            background-color: #003893;
            border-radius: 10px;
            padding: 10px 8px;
        }
        .tt-content{
            margin: 3px;
        }
        .tt-item {
            background-color: #ffd8d8;
            border-radius: 10px;
            margin: 5px 0;
            height: auto;
            padding: 5px;
            min-height: 100px;
        }
        .tt-item:hover{
            cursor: pointer;
        }
        .subject{
            font-weight: 600;
        }
        .staff{
            color: #0064d4;
            margin-top: 5px;
        }
        .bg-general {
            background-color: #e1f8ff;
        }
        .bg-marked {
            background-color: #d6ffe7;
        }
        .bg-custom {
            background-color: #e7a6ff; 
        }
        .square{
            width: 20px;
            height: 20px;
        }
    </style>
    <div class="card">
        <div class="card-body">
            <form method="get">
                <div class="row">
                    <div class="col-12 col-sm-3">
                        <div class="form-group">
                            <label for="course" class="required">Program</label>
                            <select class="form-control" id="course" name="course">
                                <option value="">Select</option>
                                @foreach($courses as $course)
                                    <option value="{{ $course->course_id }}" {{ $course->course_id==$courseId ? 'selected':'' }}>{{ $course->course_name }}</option>
                                @endforeach
                            </select>
                        </div>
                    </div>
                    <div class="col-12 col-sm-3">
                        <div class="form-group">
                            <label for="semester" class="required">Semester</label>
                            <select class="form-control" id="semester" name="semester">
                                <option value="">Select</option>
                                @foreach($semesters as $semester)
                                    <option value="{{ $semester->semester_id }}" {{ $semester->semester_id==$semesterId ? 'selected':'' }}>{{ $semester->semester_name }}</option>
                                @endforeach
                            </select>
                        </div>
                    </div>
                    <div class="col-12 col-sm-6">
                        <div class="form-group">
                            <label for="date" class="required">Date</label>
                            <div class="row">
                                <div class="col-12 col-sm-6">
                                    <div class="form-group">
                                        <input type="date" class="form-control" id="date" name="date" value="{{ $date }}" required>
                                        @error('date')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror  
                                    </div>
                                </div>
                                <div class="col-12 col-sm-6">
                                    <div class="form-group">
                                        <button type="submit" class="btn btn-sm btn-success">Submit</button>
                                    </div>
                                </div>
                            </div>   
                        </div>
                    </div>
                </div>
            </form>
        </div>
    </div>
    @if(!empty($data))
        <br>
        <div class="card">
            <div class="card-body">
                <div class="row">
                    <div class="col-12 col-sm-6">
                        <label>Date : <span class="text-danger">{{ date('d/m/Y', strtotime($date)) }}</span></label>
                    </div>
                    <div class="col-12 col-sm-6">
                        <label>Day : <span class="text-danger">{{ date('l', strtotime($date)) }}</span></label>
                    </div>
                </div>
                <div class="row">
                    <div class="col-12 col-sm-6">
                        <label>Course : <span class="text-danger">{{ $courseName }}</span></label>
                    </div>
                    <div class="col-12 col-sm-6">
                        <label>Semester : <span class="text-danger">{{ $semName }}</span></label>
                    </div>
                </div>
                <div class="row">
                    <div class="col-12 col-sm-2">
                            <div class="square bg-general"></div> General
                    </div>
                    <div class="col-12 col-sm-2">
                        <div class="square bg-custom"></div> Interchanged
                    </div>
                    <div class="col-12 col-sm-2">
                        <div class="square bg-marked"></div> Marked
                    </div>
                </div>
                <div class="timetable">
                    <div class="row">
                        @foreach($periods as $period)
                            <div class="col-12 col-sm-2">
                                @if(!empty($data[$period->id]))
                                    <div class="tt-head">Period : {{ $data[$period->id]['name'] }}</div>
                                    @if(!empty($data[$period->id]['allocation']))
                                        <div class="tt-content">
                                            @foreach($data[$period->id]['allocation'] as $detail)
                                                <div class="tt-item @if($detail['marked']>0) bg-marked @elseif($detail['type']==1) bg-custom @else bg-general @endif" @if($detail['marked']>0) onclick="alert('Attendance already marked !');" @else onclick="openDet('{{ $courseId }}','{{ $semesterId }}','{{ $date }}','{{ $period->id }}','{{ $detail['subject_id'] }}','{{ $detail['staff_id'] }}')" @endif>
                                                    <div class="subject">{{ $detail['subject_name'] }}</div>
                                                    <div class="staff">{{ $detail['staff_name'] }}</div>
                                                </div> 
                                            @endforeach
                                        </div>
                                    @else
                                        <label class="text-danger">Free Period</label>
                                    @endif
                                @endif
                            </div>
                        @endforeach
                    </div>
                </div>
            </div>
        </div>
        <div class="modal fade" id="timetableModal" tabindex="-1" role="dialog" aria-labelledby="timetableModalLabel" aria-hidden="true">
            <div class="modal-dialog" role="document">
                <div class="modal-content">
                    <div class="modal-header">
                        <h5 class="modal-title" id="timetableModalLongTitle">Timetable Interchange</h5>
                        <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                            <span aria-hidden="true">&times;</span>
                        </button>
                    </div>
                    <div class="modal-body">
                    </div>
                </div>
            </div>
        </div>
    @endif
    <script src="{{ asset('public/js/staff/general.js?ver=1.0') }}"></script>
    <script>
        function openDet(cid,sid,date,pid,sub,uid){
            var baseUrl = window.location.origin;
            
            if(cid!='' && sid!='' && date!='' && pid!='' && sub!='' && uid!='') {
                $.ajax({
                    type : 'GET',
                    url : baseUrl+'/staff-timetable-interchange-view',
                    data : { course: cid, semester: sid, date: date, period: pid, subject: sub, staff: uid },
                    success : function(response){
                        $('.modal-body').html(response);
                        $('#timetableModal').modal('show');
                    },
                    error: function (xhr, status, error) {
                        alert('Could not able to fetch data. Try again later !');
                    }
                });
            } else {
                alert('Invalid Data');
            }
        }
        
        function getSubject(cid, sid, uid) {
            var baseUrl = window.location.origin;
            
            if(cid!='' && sid!='' && uid!='') {
                $.ajax({
                    type : 'GET',
                    url : baseUrl+'/staff/get-course-subjects',
                    data : { course: cid, semester: sid, staff: uid },
                    success : function(response){
                        var data = jQuery.parseJSON(response);
                         $("#scourse").html('<option value="">Select</option>');
                        $.each(data, function(key,value) {
                            $("#scourse").append('<option value="'+key+'">'+value+'</option>');
                        }); 
                    },
                    error: function (xhr, status, error) {
                        alert('Could not able to fetch data. Try again later !');
                    }
                });
            } else {
                alert('Invalid Data');
            }
        }
    </script>
@endsection